"""Generated message classes for composer version v1alpha2.

Manages Apache Airflow environments on Google Cloud Platform.
"""
# NOTE: This file is autogenerated and should not be edited by hand.

from __future__ import absolute_import

from apitools.base.protorpclite import messages as _messages
from apitools.base.py import encoding
from apitools.base.py import extra_types


package = 'composer'


class AllowedIpRange(_messages.Message):
  r"""Allowed IP range with user-provided description.

  Fields:
    description: Optional. User-provided description. It must contain at most
      300 characters.
    value: IP address or range, defined using CIDR notation, of requests that
      this rule applies to. Examples: `192.168.1.1` or `192.168.0.0/16` or
      `2001:db8::/32` or `2001:0db8:0000:0042:0000:8a2e:0370:7334`. IP range
      prefixes should be properly truncated. For example, `1.2.3.4/24` should
      be truncated to `1.2.3.0/24`. Similarly, for IPv6, `2001:db8::1/32`
      should be truncated to `2001:db8::/32`.
  """

  description = _messages.StringField(1)
  value = _messages.StringField(2)


class CancelOperationRequest(_messages.Message):
  r"""The request message for Operations.CancelOperation."""


class CheckUpgradeRequest(_messages.Message):
  r"""Request to check whether image upgrade will succeed.

  Fields:
    imageVersion: The version of the software running in the environment. This
      encapsulates both the version of Cloud Composer functionality and the
      version of Apache Airflow. It must match the regular expression `compose
      r-([0-9]+(\.[0-9]+\.[0-9]+(-preview\.[0-9]+)?)?|latest)-airflow-([0-9]+(
      \.[0-9]+(\.[0-9]+)?)?)`. When used as input, the server also checks if
      the provided version is supported and denies the request for an
      unsupported version. The Cloud Composer portion of the image version is
      a full [semantic version](https://semver.org), or an alias in the form
      of major version number or `latest`. When an alias is provided, the
      server replaces it with the current Cloud Composer version that
      satisfies the alias. The Apache Airflow portion of the image version is
      a full semantic version that points to one of the supported Apache
      Airflow versions, or an alias in the form of only major or major.minor
      versions specified. When an alias is provided, the server replaces it
      with the latest Apache Airflow version that satisfies the alias and is
      supported in the given Cloud Composer version. In all cases, the
      resolved image version is stored in the same field. See also [version
      list](/composer/docs/concepts/versioning/composer-versions) and
      [versioning overview](/composer/docs/concepts/versioning/composer-
      versioning-overview).
  """

  imageVersion = _messages.StringField(1)


class CheckUpgradeResponse(_messages.Message):
  r"""Message containing information about the result of an upgrade check
  operation.

  Enums:
    ContainsPypiModulesConflictValueValuesEnum: Output only. Whether build has
      succeeded or failed on modules conflicts.

  Messages:
    PypiDependenciesValue: Pypi dependencies specified in the environment
      configuration, at the time when the build was triggered.

  Fields:
    buildLogUri: Output only. Url for a docker build log of an upgraded image.
    containsPypiModulesConflict: Output only. Whether build has succeeded or
      failed on modules conflicts.
    imageVersion: Composer image for which the build was happening.
    pypiConflictBuildLogExtract: Output only. Extract from a docker image
      build log containing information about pypi modules conflicts.
    pypiDependencies: Pypi dependencies specified in the environment
      configuration, at the time when the build was triggered.
  """

  class ContainsPypiModulesConflictValueValuesEnum(_messages.Enum):
    r"""Output only. Whether build has succeeded or failed on modules
    conflicts.

    Values:
      CONFLICT_RESULT_UNSPECIFIED: It is unknown whether build had conflicts
        or not.
      CONFLICT: There were python packages conflicts.
      NO_CONFLICT: There were no python packages conflicts.
    """
    CONFLICT_RESULT_UNSPECIFIED = 0
    CONFLICT = 1
    NO_CONFLICT = 2

  @encoding.MapUnrecognizedFields('additionalProperties')
  class PypiDependenciesValue(_messages.Message):
    r"""Pypi dependencies specified in the environment configuration, at the
    time when the build was triggered.

    Messages:
      AdditionalProperty: An additional property for a PypiDependenciesValue
        object.

    Fields:
      additionalProperties: Additional properties of type
        PypiDependenciesValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a PypiDependenciesValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  buildLogUri = _messages.StringField(1)
  containsPypiModulesConflict = _messages.EnumField('ContainsPypiModulesConflictValueValuesEnum', 2)
  imageVersion = _messages.StringField(3)
  pypiConflictBuildLogExtract = _messages.StringField(4)
  pypiDependencies = _messages.MessageField('PypiDependenciesValue', 5)


class CidrBlock(_messages.Message):
  r"""CIDR block with an optional name.

  Fields:
    cidrBlock: CIDR block that must be specified in CIDR notation.
    displayName: User-defined name that identifies the CIDR block.
  """

  cidrBlock = _messages.StringField(1)
  displayName = _messages.StringField(2)


class CloudDataLineageIntegration(_messages.Message):
  r"""Configuration for Cloud Data Lineage integration.

  Fields:
    enabled: Optional. Whether or not Cloud Data Lineage integration is
      enabled.
  """

  enabled = _messages.BooleanField(1)


class ComposerProjectsLocationsEnvironmentsCheckUpgradeRequest(_messages.Message):
  r"""A ComposerProjectsLocationsEnvironmentsCheckUpgradeRequest object.

  Fields:
    checkUpgradeRequest: A CheckUpgradeRequest resource to be passed as the
      request body.
    environment: The resource name of the environment to check upgrade for, in
      the form: "projects/{projectId}/locations/{locationId}/environments/{env
      ironmentId}"
  """

  checkUpgradeRequest = _messages.MessageField('CheckUpgradeRequest', 1)
  environment = _messages.StringField(2, required=True)


class ComposerProjectsLocationsEnvironmentsCreateRequest(_messages.Message):
  r"""A ComposerProjectsLocationsEnvironmentsCreateRequest object.

  Fields:
    environment: A Environment resource to be passed as the request body.
    parent: The parent must be of the form
      "projects/{projectId}/locations/{locationId}".
  """

  environment = _messages.MessageField('Environment', 1)
  parent = _messages.StringField(2, required=True)


class ComposerProjectsLocationsEnvironmentsDagsDagRunsGetRequest(_messages.Message):
  r"""A ComposerProjectsLocationsEnvironmentsDagsDagRunsGetRequest object.

  Fields:
    name: Required. The resource name of the DAG to retrieve. Must be in the
      form: "projects/{projectId}/locations/{locationId}/environments/{environ
      mentId}/dags/{dagId}/dagRuns/{dagRunId}".
  """

  name = _messages.StringField(1, required=True)


class ComposerProjectsLocationsEnvironmentsDagsDagRunsListRequest(_messages.Message):
  r"""A ComposerProjectsLocationsEnvironmentsDagsDagRunsListRequest object.

  Fields:
    filter: An expression for filtering the results. For example:
      executionDate<="2022-02-22T22:22:00Z"
    pageSize: The maximum number of DAG runs to return.
    pageToken: The next_page_token returned from a previous List request.
    parent: Required. List DAG runs in the given parent resource. Parent must
      be in the form: "projects/{projectId}/locations/{locationId}/environment
      s/{environmentId}/dags/{dagId}".
  """

  filter = _messages.StringField(1)
  pageSize = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(3)
  parent = _messages.StringField(4, required=True)


class ComposerProjectsLocationsEnvironmentsDagsDagRunsTaskInstancesListRequest(_messages.Message):
  r"""A
  ComposerProjectsLocationsEnvironmentsDagsDagRunsTaskInstancesListRequest
  object.

  Fields:
    filter: An expression for filtering the results. For example:
      executionDate<="2022-02-22T22:22:00Z"
    pageSize: The maximum number of tasks to return.
    pageToken: The next_page_token returned from a previous List request.
    parent: Required. List task instances in the given parent DAG run. Parent
      must be in the form: "projects/{projectId}/locations/{locationId}/enviro
      nments/{environmentId}/dags/{dagId}/dagRuns/{dagRunId}".
  """

  filter = _messages.StringField(1)
  pageSize = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(3)
  parent = _messages.StringField(4, required=True)


class ComposerProjectsLocationsEnvironmentsDagsGetRequest(_messages.Message):
  r"""A ComposerProjectsLocationsEnvironmentsDagsGetRequest object.

  Fields:
    name: Required. The resource name of the DAG to retrieve. Must be in the
      form: "projects/{projectId}/locations/{locationId}/environments/{environ
      mentId}/dags/{dagId}".
  """

  name = _messages.StringField(1, required=True)


class ComposerProjectsLocationsEnvironmentsDagsListRequest(_messages.Message):
  r"""A ComposerProjectsLocationsEnvironmentsDagsListRequest object.

  Fields:
    pageSize: The maximum number of DAGs to return.
    pageToken: The next_page_token returned from a previous List request.
    parent: Required. List DAGs in the given parent resource. Parent must be
      in the form: "projects/{projectId}/locations/{locationId}/environments/{
      environmentId}".
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class ComposerProjectsLocationsEnvironmentsDagsListStatsRequest(_messages.Message):
  r"""A ComposerProjectsLocationsEnvironmentsDagsListStatsRequest object.

  Fields:
    environment: Required. List DAGs in the given Composer environment. Parent
      must be in the form: "projects/{projectId}/locations/{locationId}/enviro
      nments/{environmentId}".
    interval_endTime: Optional. Exclusive end of the interval. If specified, a
      Timestamp matching this interval will have to be before the end.
    interval_startTime: Optional. Inclusive start of the interval. If
      specified, a Timestamp matching this interval will have to be the same
      or after the start.
    pageSize: The maximum number of DAGs to return.
    pageToken: The next_page_token returned from a previous List request.
  """

  environment = _messages.StringField(1, required=True)
  interval_endTime = _messages.StringField(2)
  interval_startTime = _messages.StringField(3)
  pageSize = _messages.IntegerField(4, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(5)


class ComposerProjectsLocationsEnvironmentsDagsTasksListRequest(_messages.Message):
  r"""A ComposerProjectsLocationsEnvironmentsDagsTasksListRequest object.

  Fields:
    pageSize: The maximum number of tasks to return.
    pageToken: The next_page_token returned from a previous List request.
    parent: Required. List tasks in the given parent DAG. Parent must be in
      the form: "projects/{projectId}/locations/{locationId}/environments/{env
      ironmentId}/dags/{dagId}".
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class ComposerProjectsLocationsEnvironmentsDagsTriggerRequest(_messages.Message):
  r"""A ComposerProjectsLocationsEnvironmentsDagsTriggerRequest object.

  Fields:
    dag: Required. The resource name of the DAG to trigger. Must be in the
      form: "projects/{projectId}/locations/{locationId}/environments/{environ
      mentId}/dags/{dagId}".
    triggerDagRequest: A TriggerDagRequest resource to be passed as the
      request body.
  """

  dag = _messages.StringField(1, required=True)
  triggerDagRequest = _messages.MessageField('TriggerDagRequest', 2)


class ComposerProjectsLocationsEnvironmentsDeleteRequest(_messages.Message):
  r"""A ComposerProjectsLocationsEnvironmentsDeleteRequest object.

  Fields:
    name: The environment to delete, in the form:
      "projects/{projectId}/locations/{locationId}/environments/{environmentId
      }"
  """

  name = _messages.StringField(1, required=True)


class ComposerProjectsLocationsEnvironmentsGetRequest(_messages.Message):
  r"""A ComposerProjectsLocationsEnvironmentsGetRequest object.

  Fields:
    name: The resource name of the environment to get, in the form:
      "projects/{projectId}/locations/{locationId}/environments/{environmentId
      }"
  """

  name = _messages.StringField(1, required=True)


class ComposerProjectsLocationsEnvironmentsListImportErrorsRequest(_messages.Message):
  r"""A ComposerProjectsLocationsEnvironmentsListImportErrorsRequest object.

  Fields:
    environment: Required. List import errors in the given Composer
      environment. Environment name must be in the form: "projects/{projectId}
      /locations/{locationId}/environments/{environmentId}".
    pageSize: The maximum number of DAGs to return.
    pageToken: The next_page_token returned from a previous List request.
  """

  environment = _messages.StringField(1, required=True)
  pageSize = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(3)


class ComposerProjectsLocationsEnvironmentsListRequest(_messages.Message):
  r"""A ComposerProjectsLocationsEnvironmentsListRequest object.

  Fields:
    pageSize: The maximum number of environments to return.
    pageToken: The next_page_token value returned from a previous List
      request, if any.
    parent: List environments in the given project and location, in the form:
      "projects/{projectId}/locations/{locationId}"
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class ComposerProjectsLocationsEnvironmentsLoadSnapshotRequest(_messages.Message):
  r"""A ComposerProjectsLocationsEnvironmentsLoadSnapshotRequest object.

  Fields:
    environment: The resource name of the target environment in the form:
      "projects/{projectId}/locations/{locationId}/environments/{environmentId
      }"
    loadSnapshotRequest: A LoadSnapshotRequest resource to be passed as the
      request body.
  """

  environment = _messages.StringField(1, required=True)
  loadSnapshotRequest = _messages.MessageField('LoadSnapshotRequest', 2)


class ComposerProjectsLocationsEnvironmentsPatchRequest(_messages.Message):
  r"""A ComposerProjectsLocationsEnvironmentsPatchRequest object.

  Fields:
    environment: A Environment resource to be passed as the request body.
    name: The relative resource name of the environment to update, in the
      form: "projects/{projectId}/locations/{locationId}/environments/{environ
      mentId}"
    updateMask: Required. A comma-separated list of paths, relative to
      `Environment`, of fields to update. For example, to set the version of
      scikit-learn to install in the environment to 0.19.0 and to remove an
      existing installation of argparse, the `updateMask` parameter would
      include the following two `paths` values:
      "config.softwareConfig.pypiPackages.scikit-learn" and
      "config.softwareConfig.pypiPackages.argparse". The included patch
      environment would specify the scikit-learn version as follows: {
      "config":{ "softwareConfig":{ "pypiPackages":{ "scikit-learn":"==0.19.0"
      } } } } Note that in the above example, any existing PyPI packages other
      than scikit-learn and argparse will be unaffected. Only one update type
      may be included in a single request's `updateMask`. For example, one
      cannot update both the PyPI packages and labels in the same request.
      However, it is possible to update multiple members of a map field
      simultaneously in the same request. For example, to set the labels
      "label1" and "label2" while clearing "label3" (assuming it already
      exists), one can provide the paths "labels.label1", "labels.label2", and
      "labels.label3" and populate the patch environment as follows: {
      "labels":{ "label1":"new-label1-value" "label2":"new-label2-value" } }
      Note that in the above example, any existing labels that are not
      included in the `updateMask` will be unaffected. It is also possible to
      replace an entire map field by providing the map field's path in the
      `updateMask`. The new value of the field will be that which is provided
      in the patch environment. For example, to delete all pre-existing user-
      specified PyPI packages and install botocore at version 1.7.14, the
      `updateMask` would contain the path
      "config.softwareConfig.pypiPackages", and the patch environment would be
      the following: { "config":{ "softwareConfig":{ "pypiPackages":{
      "botocore":"==1.7.14" } } } } **Note:** Only the following fields can be
      updated: * `config.softwareConfig.pypiPackages` * Replace all custom
      custom PyPI packages. If a replacement package map is not included in
      `environment`, all custom PyPI packages are cleared. It is an error to
      provide both this mask and a mask specifying an individual package. *
      `config.softwareConfig.pypiPackages.`packagename * Update the custom
      PyPI package *packagename*, preserving other packages. To delete the
      package, include it in `updateMask`, and omit the mapping for it in
      `environment.config.softwareConfig.pypiPackages`. It is an error to
      provide both a mask of this form and the
      `config.softwareConfig.pypiPackages` mask. * `labels` * Replace all
      environment labels. If a replacement labels map is not included in
      `environment`, all labels are cleared. It is an error to provide both
      this mask and a mask specifying one or more individual labels. *
      `labels.`labelName * Set the label named *labelName*, while preserving
      other labels. To delete the label, include it in `updateMask` and omit
      its mapping in `environment.labels`. It is an error to provide both a
      mask of this form and the `labels` mask. * `config.nodeCount` *
      Horizontally scale the number of nodes in the environment. An integer
      greater than or equal to 3 must be provided in the `config.nodeCount`
      field. Supported for Cloud Composer environments in versions
      composer-1.*.*-airflow-*.*.*. * `config.webServerNetworkAccessControl` *
      Replace the environment's current `WebServerNetworkAccessControl`. *
      `config.softwareConfig.airflowConfigOverrides` * Replace all Apache
      Airflow config overrides. If a replacement config overrides map is not
      included in `environment`, all config overrides are cleared. It is an
      error to provide both this mask and a mask specifying one or more
      individual config overrides. *
      `config.softwareConfig.airflowConfigOverrides.`section-name * Override
      the Apache Airflow config property *name* in the section named
      *section*, preserving other properties. To delete the property override,
      include it in `updateMask` and omit its mapping in
      `environment.config.softwareConfig.airflowConfigOverrides`. It is an
      error to provide both a mask of this form and the
      `config.softwareConfig.airflowConfigOverrides` mask. *
      `config.softwareConfig.envVariables` * Replace all environment
      variables. If a replacement environment variable map is not included in
      `environment`, all custom environment variables are cleared. *
      `config.softwareConfig.imageVersion` * Upgrade the version of the
      environment in-place. Refer to `SoftwareConfig.image_version` for
      information on how to format the new image version. Additionally, the
      new image version cannot effect a version downgrade, and must match the
      current image version's Composer and Airflow major versions. Consult the
      [Cloud Composer version
      list](/composer/docs/concepts/versioning/composer-versions) for valid
      values. * `config.softwareConfig.schedulerCount` * Horizontally scale
      the number of schedulers in Airflow. A positive integer not greater than
      the number of nodes must be provided in the
      `config.softwareConfig.schedulerCount` field. Supported for Cloud
      Composer environments in versions composer-1.*.*-airflow-2.*.*. *
      `config.softwareConfig.cloudDataLineageIntegration` * Configuration for
      Cloud Data Lineage integration. * `config.databaseConfig.machineType` *
      Cloud SQL machine type used by Airflow database. It has to be one of:
      db-n1-standard-2, db-n1-standard-4, db-n1-standard-8 or
      db-n1-standard-16. Supported for Cloud Composer environments in versions
      composer-1.*.*-airflow-*.*.*. * `config.webServerConfig.machineType` *
      Machine type on which Airflow web server is running. It has to be one
      of: composer-n1-webserver-2, composer-n1-webserver-4 or
      composer-n1-webserver-8. Supported for Cloud Composer environments in
      versions composer-1.*.*-airflow-*.*.*. * `config.maintenanceWindow` *
      Maintenance window during which Cloud Composer components may be under
      maintenance. * `config.workloadsConfig` * The workloads configuration
      settings for the GKE cluster associated with the Cloud Composer
      environment. Supported for Cloud Composer environments in versions
      composer-2.*.*-airflow-*.*.* and newer. * `config.environmentSize` * The
      size of the Cloud Composer environment. Supported for Cloud Composer
      environments in versions composer-2.*.*-airflow-*.*.* and newer.
  """

  environment = _messages.MessageField('Environment', 1)
  name = _messages.StringField(2, required=True)
  updateMask = _messages.StringField(3)


class ComposerProjectsLocationsEnvironmentsRestartWebServerRequest(_messages.Message):
  r"""A ComposerProjectsLocationsEnvironmentsRestartWebServerRequest object.

  Fields:
    name: The resource name of the environment to restart the web server for,
      in the form: "projects/{projectId}/locations/{locationId}/environments/{
      environmentId}"
    restartWebServerRequest: A RestartWebServerRequest resource to be passed
      as the request body.
  """

  name = _messages.StringField(1, required=True)
  restartWebServerRequest = _messages.MessageField('RestartWebServerRequest', 2)


class ComposerProjectsLocationsEnvironmentsSaveSnapshotRequest(_messages.Message):
  r"""A ComposerProjectsLocationsEnvironmentsSaveSnapshotRequest object.

  Fields:
    environment: The resource name of the source environment in the form:
      "projects/{projectId}/locations/{locationId}/environments/{environmentId
      }"
    saveSnapshotRequest: A SaveSnapshotRequest resource to be passed as the
      request body.
  """

  environment = _messages.StringField(1, required=True)
  saveSnapshotRequest = _messages.MessageField('SaveSnapshotRequest', 2)


class ComposerProjectsLocationsImageVersionsListRequest(_messages.Message):
  r"""A ComposerProjectsLocationsImageVersionsListRequest object.

  Fields:
    includePastReleases: Whether or not image versions from old releases
      should be included.
    pageSize: The maximum number of image_versions to return.
    pageToken: The next_page_token value returned from a previous List
      request, if any.
    parent: List ImageVersions in the given project and location, in the form:
      "projects/{projectId}/locations/{locationId}"
  """

  includePastReleases = _messages.BooleanField(1)
  pageSize = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(3)
  parent = _messages.StringField(4, required=True)


class ComposerProjectsLocationsOperationsCancelRequest(_messages.Message):
  r"""A ComposerProjectsLocationsOperationsCancelRequest object.

  Fields:
    cancelOperationRequest: A CancelOperationRequest resource to be passed as
      the request body.
    name: The name of the operation resource to be cancelled.
  """

  cancelOperationRequest = _messages.MessageField('CancelOperationRequest', 1)
  name = _messages.StringField(2, required=True)


class ComposerProjectsLocationsOperationsDeleteRequest(_messages.Message):
  r"""A ComposerProjectsLocationsOperationsDeleteRequest object.

  Fields:
    name: The name of the operation resource to be deleted.
  """

  name = _messages.StringField(1, required=True)


class ComposerProjectsLocationsOperationsGetRequest(_messages.Message):
  r"""A ComposerProjectsLocationsOperationsGetRequest object.

  Fields:
    name: The name of the operation resource.
  """

  name = _messages.StringField(1, required=True)


class ComposerProjectsLocationsOperationsListRequest(_messages.Message):
  r"""A ComposerProjectsLocationsOperationsListRequest object.

  Fields:
    filter: The standard list filter.
    name: The name of the operation's parent resource.
    pageSize: The standard list page size.
    pageToken: The standard list page token.
  """

  filter = _messages.StringField(1)
  name = _messages.StringField(2, required=True)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)


class Dag(_messages.Message):
  r"""A Composer DAG resource.

  Enums:
    StateValueValuesEnum: Output only. The current state of the DAG.

  Fields:
    cronSchedule: The DAG's schedule in cron format.
    dagId: Required. The DAG ID.
    description: The description of the DAG.
    durationSchedule: The DAG's schedule as a time duration between runs.
    fileloc: File location relative to the Cloud Storage bucket root folder.
    lastRunEndTime: The end timestamp of the last completed DAG run.
    name: Required. The resource name of the DAG, in the form: "projects/{proj
      ectId}/locations/{locationId}/environments/{environmentId}/dags/{dagId}"
      .
    runningCount: The number of running instances of the DAG.
    state: Output only. The current state of the DAG.
  """

  class StateValueValuesEnum(_messages.Enum):
    r"""Output only. The current state of the DAG.

    Values:
      STATE_UNSPECIFIED: The state of the DAG is unknown.
      ACTIVE: The DAG is available for execution.
      PAUSED: The DAG is paused.
    """
    STATE_UNSPECIFIED = 0
    ACTIVE = 1
    PAUSED = 2

  cronSchedule = _messages.StringField(1)
  dagId = _messages.StringField(2)
  description = _messages.StringField(3)
  durationSchedule = _messages.StringField(4)
  fileloc = _messages.StringField(5)
  lastRunEndTime = _messages.StringField(6)
  name = _messages.StringField(7)
  runningCount = _messages.IntegerField(8, variant=_messages.Variant.INT32)
  state = _messages.EnumField('StateValueValuesEnum', 9)


class DagRun(_messages.Message):
  r"""A single DAG run.

  Enums:
    StateValueValuesEnum: DAG run state.

  Fields:
    dagId: The DAG ID of the DAG whose execution is described by this DAG run.
    dagRunId: The DAG run ID.
    endDate: Timestamp when the DAG run ended. Set only if the DAG run has
      finished.
    executionDate: The logical date and time which the DAG run and its task
      instances are running for.
    name: The resource name of the DAG, in the form: "projects/{projectId}/loc
      ations/{locationId}/environments/{environmentId}/dags/{dagId}/dagRuns/{d
      agRunId}".
    startDate: Timestamp when the DAG run started.
    state: DAG run state.
  """

  class StateValueValuesEnum(_messages.Enum):
    r"""DAG run state.

    Values:
      STATE_UNSPECIFIED: The state of the DAG run is unknown.
      RUNNING: The DAG run is being executed.
      SUCCEEDED: The DAG run is finished successfully.
      FAILED: The DAG run is finished with an error.
      QUEUED: The DAG run is queued for execution.
    """
    STATE_UNSPECIFIED = 0
    RUNNING = 1
    SUCCEEDED = 2
    FAILED = 3
    QUEUED = 4

  dagId = _messages.StringField(1)
  dagRunId = _messages.StringField(2)
  endDate = _messages.StringField(3)
  executionDate = _messages.StringField(4)
  name = _messages.StringField(5)
  startDate = _messages.StringField(6)
  state = _messages.EnumField('StateValueValuesEnum', 7)


class DagStats(_messages.Message):
  r"""Statistics of a DAG in a specific time interval.

  Fields:
    dag: DAG name.
    failedRunCount: Number of DAG runs finished with a failure in the time
      interval.
    successfulRunCount: Number of DAG runs successfully finished in the time
      interval.
  """

  dag = _messages.StringField(1)
  failedRunCount = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  successfulRunCount = _messages.IntegerField(3, variant=_messages.Variant.INT32)


class DataRetentionConfig(_messages.Message):
  r"""The configuration setting for Airflow database data retention mechanism.

  Fields:
    airflowDatabaseRetentionDays: Optional. The number of days describing for
      how long to store event-based records in airflow database. If the
      retention mechanism is enabled this value must be a positive integer
      otherwise, value should be set to 0.
  """

  airflowDatabaseRetentionDays = _messages.IntegerField(1, variant=_messages.Variant.INT32)


class DatabaseConfig(_messages.Message):
  r"""The configuration of Cloud SQL instance that is used by the Apache
  Airflow software.

  Fields:
    highAvailability: Optional. Creates the Airflow Database in High
      Availability mode. This option can only be set during environment
      creation.
    machineType: Optional. Cloud SQL machine type used by Airflow database. It
      has to be one of: db-n1-standard-2, db-n1-standard-4, db-n1-standard-8
      or db-n1-standard-16. If not specified, db-n1-standard-2 will be used.
      Supported for Cloud Composer environments in versions
      composer-1.*.*-airflow-*.*.*.
    zone: Optional. The Compute Engine zone where the Airflow database is
      created. If zone is provided, it must be in the region selected for the
      environment. If zone is not provided, a zone is automatically selected.
      The zone can only be set during environment creation. Supported for
      Cloud Composer environments in versions composer-2.*.*-airflow-*.*.*.
  """

  highAvailability = _messages.BooleanField(1)
  machineType = _messages.StringField(2)
  zone = _messages.StringField(3)


class Date(_messages.Message):
  r"""Represents a whole or partial calendar date, such as a birthday. The
  time of day and time zone are either specified elsewhere or are
  insignificant. The date is relative to the Gregorian Calendar. This can
  represent one of the following: * A full date, with non-zero year, month,
  and day values. * A month and day, with a zero year (for example, an
  anniversary). * A year on its own, with a zero month and a zero day. * A
  year and month, with a zero day (for example, a credit card expiration
  date). Related types: * google.type.TimeOfDay * google.type.DateTime *
  google.protobuf.Timestamp

  Fields:
    day: Day of a month. Must be from 1 to 31 and valid for the year and
      month, or 0 to specify a year by itself or a year and month where the
      day isn't significant.
    month: Month of a year. Must be from 1 to 12, or 0 to specify a year
      without a month and day.
    year: Year of the date. Must be from 1 to 9999, or 0 to specify a date
      without a year.
  """

  day = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  month = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  year = _messages.IntegerField(3, variant=_messages.Variant.INT32)


class Empty(_messages.Message):
  r"""A generic empty message that you can re-use to avoid defining duplicated
  empty messages in your APIs. A typical example is to use it as the request
  or the response type of an API method. For instance: service Foo { rpc
  Bar(google.protobuf.Empty) returns (google.protobuf.Empty); }
  """



class EncryptionConfig(_messages.Message):
  r"""The encryption options for the Cloud Composer environment and its
  dependencies. Supported for Cloud Composer environments in versions
  composer-1.*.*-airflow-*.*.*.

  Fields:
    kmsKeyName: Optional. Customer-managed Encryption Key available through
      Google's Key Management Service. Cannot be updated. If not specified,
      Google-managed key will be used.
  """

  kmsKeyName = _messages.StringField(1)


class Environment(_messages.Message):
  r"""An environment for running orchestration tasks.

  Enums:
    StateValueValuesEnum: The current state of the environment.

  Messages:
    LabelsValue: Optional. User-defined labels for this environment. The
      labels map can contain no more than 64 entries. Entries of the labels
      map are UTF8 strings that comply with the following restrictions: * Keys
      must conform to regexp: \p{Ll}\p{Lo}{0,62} * Values must conform to
      regexp: [\p{Ll}\p{Lo}\p{N}_-]{0,63} * Both keys and values are
      additionally constrained to be <= 128 bytes in size.

  Fields:
    config: Configuration parameters for this environment.
    createTime: Output only. The time at which this environment was created.
    labels: Optional. User-defined labels for this environment. The labels map
      can contain no more than 64 entries. Entries of the labels map are UTF8
      strings that comply with the following restrictions: * Keys must conform
      to regexp: \p{Ll}\p{Lo}{0,62} * Values must conform to regexp:
      [\p{Ll}\p{Lo}\p{N}_-]{0,63} * Both keys and values are additionally
      constrained to be <= 128 bytes in size.
    name: The resource name of the environment, in the form:
      "projects/{projectId}/locations/{locationId}/environments/{environmentId
      }" EnvironmentId must start with a lowercase letter followed by up to 63
      lowercase letters, numbers, or hyphens, and cannot end with a hyphen.
    satisfiesPzs: Output only. Reserved for future use.
    state: The current state of the environment.
    updateTime: Output only. The time at which this environment was last
      modified.
    uuid: Output only. The UUID (Universally Unique IDentifier) associated
      with this environment. This value is generated when the environment is
      created.
  """

  class StateValueValuesEnum(_messages.Enum):
    r"""The current state of the environment.

    Values:
      STATE_UNSPECIFIED: The state of the environment is unknown.
      CREATING: The environment is in the process of being created.
      RUNNING: The environment is currently running and healthy. It is ready
        for use.
      UPDATING: The environment is being updated. It remains usable but cannot
        receive additional update requests or be deleted at this time.
      DELETING: The environment is undergoing deletion. It cannot be used.
      ERROR: The environment has encountered an error and cannot be used.
    """
    STATE_UNSPECIFIED = 0
    CREATING = 1
    RUNNING = 2
    UPDATING = 3
    DELETING = 4
    ERROR = 5

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Optional. User-defined labels for this environment. The labels map can
    contain no more than 64 entries. Entries of the labels map are UTF8
    strings that comply with the following restrictions: * Keys must conform
    to regexp: \p{Ll}\p{Lo}{0,62} * Values must conform to regexp:
    [\p{Ll}\p{Lo}\p{N}_-]{0,63} * Both keys and values are additionally
    constrained to be <= 128 bytes in size.

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  config = _messages.MessageField('EnvironmentConfig', 1)
  createTime = _messages.StringField(2)
  labels = _messages.MessageField('LabelsValue', 3)
  name = _messages.StringField(4)
  satisfiesPzs = _messages.BooleanField(5)
  state = _messages.EnumField('StateValueValuesEnum', 6)
  updateTime = _messages.StringField(7)
  uuid = _messages.StringField(8)


class EnvironmentConfig(_messages.Message):
  r"""Configuration information for an environment.

  Enums:
    EnvironmentSizeValueValuesEnum: Optional. The size of the Cloud Composer
      environment. This field is supported for Cloud Composer environments in
      versions composer-2.*.*-airflow-*.*.* and newer.

  Fields:
    airflowUri: Output only. The URI of the Apache Airflow Web UI hosted
      within this environment (see [Airflow web interface](/composer/docs/how-
      to/accessing/airflow-web-interface)).
    dagGcsPrefix: Output only. The Cloud Storage prefix of the DAGs for this
      environment. Although Cloud Storage objects reside in a flat namespace,
      a hierarchical file tree can be simulated using "/"-delimited object
      name prefixes. DAG objects for this environment reside in a simulated
      directory with the given prefix.
    dataRetentionConfig: Optional. The configuration setting for Airflow
      database data retention mechanism.
    databaseConfig: Optional. The configuration settings for Cloud SQL
      instance used internally by Apache Airflow software.
    encryptionConfig: Optional. The encryption options for the Cloud Composer
      environment and its dependencies. Cannot be updated.
    environmentSize: Optional. The size of the Cloud Composer environment.
      This field is supported for Cloud Composer environments in versions
      composer-2.*.*-airflow-*.*.* and newer.
    gkeCluster: Output only. The Kubernetes Engine cluster used to run this
      environment.
    maintenanceWindow: Optional. The maintenance window is the period when
      Cloud Composer components may undergo maintenance. It is defined so that
      maintenance is not executed during peak hours or critical time periods.
      The system will not be under maintenance for every occurrence of this
      window, but when maintenance is planned, it will be scheduled during the
      window. The maintenance window period must encompass at least 12 hours
      per week. This may be split into multiple chunks, each with a size of at
      least 4 hours. If this value is omitted, Cloud Composer components may
      be subject to maintenance at any time.
    masterAuthorizedNetworksConfig: Optional. The configuration options for
      GKE cluster master authorized networks. By default master authorized
      networks feature is: - in case of private environment: enabled with no
      external networks allowlisted. - in case of public environment:
      disabled.
    nodeConfig: The configuration used for the Kubernetes Engine cluster.
    nodeCount: The number of nodes in the Kubernetes Engine cluster that will
      be used to run this environment. This field is supported for Cloud
      Composer environments in versions composer-1.*.*-airflow-*.*.*.
    privateEnvironmentConfig: The configuration used for the Private IP Cloud
      Composer environment.
    recoveryConfig: Optional. The Recovery settings configuration of an
      environment. This field is supported for Cloud Composer environments in
      versions composer-2.*.*-airflow-*.*.* and newer.
    softwareConfig: The configuration settings for software inside the
      environment.
    webServerConfig: Optional. The configuration settings for the Airflow web
      server App Engine instance. This field is supported for Cloud Composer
      environments in versions composer-1.*.*-airflow-*.*.*.
    webServerNetworkAccessControl: Optional. The network-level access control
      policy for the Airflow web server. If unspecified, no network-level
      access restrictions will be applied.
    workloadsConfig: Optional. The workloads configuration settings for the
      GKE cluster associated with the Cloud Composer environment. The GKE
      cluster runs Airflow scheduler and workers workloads. This field is
      supported for Cloud Composer environments in versions
      composer-2.*.*-airflow-*.*.* and newer.
  """

  class EnvironmentSizeValueValuesEnum(_messages.Enum):
    r"""Optional. The size of the Cloud Composer environment. This field is
    supported for Cloud Composer environments in versions
    composer-2.*.*-airflow-*.*.* and newer.

    Values:
      ENVIRONMENT_SIZE_UNSPECIFIED: The size of the environment is
        unspecified.
      ENVIRONMENT_SIZE_SMALL: The environment size is small.
      ENVIRONMENT_SIZE_MEDIUM: The environment size is medium.
      ENVIRONMENT_SIZE_LARGE: The environment size is large.
    """
    ENVIRONMENT_SIZE_UNSPECIFIED = 0
    ENVIRONMENT_SIZE_SMALL = 1
    ENVIRONMENT_SIZE_MEDIUM = 2
    ENVIRONMENT_SIZE_LARGE = 3

  airflowUri = _messages.StringField(1)
  dagGcsPrefix = _messages.StringField(2)
  dataRetentionConfig = _messages.MessageField('DataRetentionConfig', 3)
  databaseConfig = _messages.MessageField('DatabaseConfig', 4)
  encryptionConfig = _messages.MessageField('EncryptionConfig', 5)
  environmentSize = _messages.EnumField('EnvironmentSizeValueValuesEnum', 6)
  gkeCluster = _messages.StringField(7)
  maintenanceWindow = _messages.MessageField('MaintenanceWindow', 8)
  masterAuthorizedNetworksConfig = _messages.MessageField('MasterAuthorizedNetworksConfig', 9)
  nodeConfig = _messages.MessageField('NodeConfig', 10)
  nodeCount = _messages.IntegerField(11, variant=_messages.Variant.INT32)
  privateEnvironmentConfig = _messages.MessageField('PrivateEnvironmentConfig', 12)
  recoveryConfig = _messages.MessageField('RecoveryConfig', 13)
  softwareConfig = _messages.MessageField('SoftwareConfig', 14)
  webServerConfig = _messages.MessageField('WebServerConfig', 15)
  webServerNetworkAccessControl = _messages.MessageField('WebServerNetworkAccessControl', 16)
  workloadsConfig = _messages.MessageField('WorkloadsConfig', 17)


class IPAllocationPolicy(_messages.Message):
  r"""Configuration for controlling how IPs are allocated in the GKE cluster
  running the Apache Airflow software.

  Fields:
    clusterIpv4CidrBlock: Optional. The IP address range used to allocate IP
      addresses to pods in the GKE cluster. For Cloud Composer environments in
      versions composer-1.*.*-airflow-*.*.*, this field is applicable only
      when `use_ip_aliases` is true. Set to blank to have GKE choose a range
      with the default size. Set to /netmask (e.g. `/14`) to have GKE choose a
      range with a specific netmask. Set to a
      [CIDR](https://en.wikipedia.org/wiki/Classless_Inter-Domain_Routing)
      notation (e.g. `10.96.0.0/14`) from the RFC-1918 private networks (e.g.
      `10.0.0.0/8`, `172.16.0.0/12`, `192.168.0.0/16`) to pick a specific
      range to use.
    clusterSecondaryRangeName: Optional. The name of the GKE cluster's
      secondary range used to allocate IP addresses to pods. For Cloud
      Composer environments in versions composer-1.*.*-airflow-*.*.*, this
      field is applicable only when `use_ip_aliases` is true.
    servicesIpv4CidrBlock: Optional. The IP address range of the services IP
      addresses in this GKE cluster. For Cloud Composer environments in
      versions composer-1.*.*-airflow-*.*.*, this field is applicable only
      when `use_ip_aliases` is true. Set to blank to have GKE choose a range
      with the default size. Set to /netmask (e.g. `/14`) to have GKE choose a
      range with a specific netmask. Set to a
      [CIDR](https://en.wikipedia.org/wiki/Classless_Inter-Domain_Routing)
      notation (e.g. `10.96.0.0/14`) from the RFC-1918 private networks (e.g.
      `10.0.0.0/8`, `172.16.0.0/12`, `192.168.0.0/16`) to pick a specific
      range to use.
    servicesSecondaryRangeName: Optional. The name of the services' secondary
      range used to allocate IP addresses to the GKE cluster. For Cloud
      Composer environments in versions composer-1.*.*-airflow-*.*.*, this
      field is applicable only when `use_ip_aliases` is true.
    useIpAliases: Optional. Whether or not to enable Alias IPs in the GKE
      cluster. If `true`, a VPC-native cluster is created. This field is only
      supported for Cloud Composer environments in versions
      composer-1.*.*-airflow-*.*.*. Environments in newer versions always use
      VPC-native GKE clusters.
  """

  clusterIpv4CidrBlock = _messages.StringField(1)
  clusterSecondaryRangeName = _messages.StringField(2)
  servicesIpv4CidrBlock = _messages.StringField(3)
  servicesSecondaryRangeName = _messages.StringField(4)
  useIpAliases = _messages.BooleanField(5)


class ImageVersion(_messages.Message):
  r"""Image Version information

  Fields:
    creationDisabled: Whether it is impossible to create an environment with
      the image version.
    imageVersionId: The string identifier of the ImageVersion, in the form:
      "composer-x.y.z-airflow-a.b.c"
    isDefault: Whether this is the default ImageVersion used by Composer
      during environment creation if no input ImageVersion is specified.
    releaseDate: The date of the version release.
    supportedPythonVersions: supported python versions
    upgradeDisabled: Whether it is impossible to upgrade an environment
      running with the image version.
  """

  creationDisabled = _messages.BooleanField(1)
  imageVersionId = _messages.StringField(2)
  isDefault = _messages.BooleanField(3)
  releaseDate = _messages.MessageField('Date', 4)
  supportedPythonVersions = _messages.StringField(5, repeated=True)
  upgradeDisabled = _messages.BooleanField(6)


class ImportError(_messages.Message):
  r"""DAG import error details.

  Fields:
    creationTime: Time when the error was created.
    filename: Location of the file with the error relative to the Cloud
      Storage bucket root folder.
    stackTrace: Full stack trace.
  """

  creationTime = _messages.StringField(1)
  filename = _messages.StringField(2)
  stackTrace = _messages.StringField(3)


class ListDagRunsResponse(_messages.Message):
  r"""Response to `ListDagRunsRequest`.

  Fields:
    dagRuns: The list of DAG runs returned.
    nextPageToken: The page token used to query for the next page if one
      exists.
  """

  dagRuns = _messages.MessageField('DagRun', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class ListDagStatsResponse(_messages.Message):
  r"""Response to `ListDagStatsRequest`.

  Fields:
    dagStats: List of DAGs with statistics.
    nextPageToken: The page token used to query for the next page if one
      exists.
  """

  dagStats = _messages.MessageField('DagStats', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class ListDagsResponse(_messages.Message):
  r"""Response to `ListDagsRequest`.

  Fields:
    dags: The list of DAGs returned.
    nextPageToken: The page token used to query for the next page if one
      exists.
  """

  dags = _messages.MessageField('Dag', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class ListEnvironmentsResponse(_messages.Message):
  r"""The environments in a project and location.

  Fields:
    environments: The list of environments returned by a
      ListEnvironmentsRequest.
    nextPageToken: The page token used to query for the next page if one
      exists.
  """

  environments = _messages.MessageField('Environment', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class ListImageVersionsResponse(_messages.Message):
  r"""The ImageVersions in a project and location.

  Fields:
    imageVersions: The list of supported ImageVersions in a location.
    nextPageToken: The page token used to query for the next page if one
      exists.
  """

  imageVersions = _messages.MessageField('ImageVersion', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class ListImportErrorsResponse(_messages.Message):
  r"""Response to `ListImportErrorsRequest`.

  Fields:
    importErrors: List of DAGs with statistics.
    nextPageToken: The page token used to query for the next page if one
      exists.
  """

  importErrors = _messages.MessageField('ImportError', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class ListOperationsResponse(_messages.Message):
  r"""The response message for Operations.ListOperations.

  Fields:
    nextPageToken: The standard List next-page token.
    operations: A list of operations that matches the specified filter in the
      request.
  """

  nextPageToken = _messages.StringField(1)
  operations = _messages.MessageField('Operation', 2, repeated=True)


class ListTaskInstancesResponse(_messages.Message):
  r"""Response to `ListTaskInstancesRequest`.

  Fields:
    nextPageToken: The page token used to query for the next page if one
      exists.
    taskInstances: The list of tasks returned.
  """

  nextPageToken = _messages.StringField(1)
  taskInstances = _messages.MessageField('TaskInstance', 2, repeated=True)


class ListTasksResponse(_messages.Message):
  r"""Response to `ListTasksRequest`.

  Fields:
    nextPageToken: The page token used to query for the next page if one
      exists.
    tasks: The list of tasks returned.
  """

  nextPageToken = _messages.StringField(1)
  tasks = _messages.MessageField('Task', 2, repeated=True)


class LoadSnapshotRequest(_messages.Message):
  r"""Request to load a snapshot into a Cloud Composer environment.

  Fields:
    skipAirflowOverridesSetting: Whether or not to skip setting Airflow
      overrides when loading the environment's state.
    skipEnvironmentVariablesSetting: Whether or not to skip setting
      environment variables when loading the environment's state.
    skipGcsDataCopying: Whether or not to skip copying Cloud Storage data when
      loading the environment's state.
    skipPypiPackagesInstallation: Whether or not to skip installing Pypi
      packages when loading the environment's state.
    snapshotPath: A Cloud Storage path to a snapshot to load, e.g.: "gs://my-
      bucket/snapshots/project_location_environment_timestamp".
  """

  skipAirflowOverridesSetting = _messages.BooleanField(1)
  skipEnvironmentVariablesSetting = _messages.BooleanField(2)
  skipGcsDataCopying = _messages.BooleanField(3)
  skipPypiPackagesInstallation = _messages.BooleanField(4)
  snapshotPath = _messages.StringField(5)


class LoadSnapshotResponse(_messages.Message):
  r"""Response to LoadSnapshotRequest."""


class MaintenanceWindow(_messages.Message):
  r"""The configuration settings for Cloud Composer maintenance window. The
  following example: ``` { "startTime":"2019-08-01T01:00:00Z"
  "endTime":"2019-08-01T07:00:00Z" "recurrence":"FREQ=WEEKLY;BYDAY=TU,WE" }
  ``` would define a maintenance window between 01 and 07 hours UTC during
  each Tuesday and Wednesday.

  Fields:
    endTime: Required. Maintenance window end time. It is used only to
      calculate the duration of the maintenance window. The value for end_time
      must be in the future, relative to `start_time`.
    recurrence: Required. Maintenance window recurrence. Format is a subset of
      [RFC-5545](https://tools.ietf.org/html/rfc5545) `RRULE`. The only
      allowed values for `FREQ` field are `FREQ=DAILY` and
      `FREQ=WEEKLY;BYDAY=...` Example values: `FREQ=WEEKLY;BYDAY=TU,WE`,
      `FREQ=DAILY`.
    startTime: Required. Start time of the first recurrence of the maintenance
      window.
  """

  endTime = _messages.StringField(1)
  recurrence = _messages.StringField(2)
  startTime = _messages.StringField(3)


class MasterAuthorizedNetworksConfig(_messages.Message):
  r"""Configuration options for the master authorized networks feature.
  Enabled master authorized networks will disallow all external traffic to
  access Kubernetes master through HTTPS except traffic from the given CIDR
  blocks, Google Compute Engine Public IPs and Google Prod IPs.

  Fields:
    cidrBlocks: Up to 50 external networks that could access Kubernetes master
      through HTTPS.
    enabled: Whether or not master authorized networks feature is enabled.
  """

  cidrBlocks = _messages.MessageField('CidrBlock', 1, repeated=True)
  enabled = _messages.BooleanField(2)


class NodeConfig(_messages.Message):
  r"""The configuration information for the Kubernetes Engine nodes running
  the Apache Airflow software.

  Fields:
    diskSizeGb: Optional. The disk size in GB used for node VMs. Minimum size
      is 30GB. If unspecified, defaults to 100GB. Cannot be updated. This
      field is supported for Cloud Composer environments in versions
      composer-1.*.*-airflow-*.*.*.
    enableIpMasqAgent: Optional. Deploys 'ip-masq-agent' daemon set in the GKE
      cluster and defines nonMasqueradeCIDRs equals to pod IP range so IP
      masquerading is used for all destination addresses, except between pods
      traffic. See: https://cloud.google.com/kubernetes-engine/docs/how-to/ip-
      masquerade-agent
    ipAllocationPolicy: Optional. The IPAllocationPolicy fields for the GKE
      cluster.
    location: Optional. The Compute Engine [zone](/compute/docs/regions-zones)
      in which to deploy the VMs used to run the Apache Airflow software,
      specified as a [relative resource
      name](/apis/design/resource_names#relative_resource_name). For example:
      "projects/{projectId}/zones/{zoneId}". This `location` must belong to
      the enclosing environment's project and location. If both this field and
      `nodeConfig.machineType` are specified, `nodeConfig.machineType` must
      belong to this `location`; if both are unspecified, the service will
      pick a zone in the Compute Engine region corresponding to the Cloud
      Composer location, and propagate that choice to both fields. If only one
      field (`location` or `nodeConfig.machineType`) is specified, the
      location information from the specified field will be propagated to the
      unspecified field. This field is supported for Cloud Composer
      environments in versions composer-1.*.*-airflow-*.*.*.
    machineType: Optional. The Compute Engine [machine
      type](/compute/docs/machine-types) used for cluster instances, specified
      as a [relative resource
      name](/apis/design/resource_names#relative_resource_name). For example:
      "projects/{projectId}/zones/{zoneId}/machineTypes/{machineTypeId}". The
      `machineType` must belong to the enclosing environment's project and
      location. If both this field and `nodeConfig.location` are specified,
      this `machineType` must belong to the `nodeConfig.location`; if both are
      unspecified, the service will pick a zone in the Compute Engine region
      corresponding to the Cloud Composer location, and propagate that choice
      to both fields. If exactly one of this field and `nodeConfig.location`
      is specified, the location information from the specified field will be
      propagated to the unspecified field. The `machineTypeId` must not be a
      [shared-core machine type](/compute/docs/machine-types#sharedcore). If
      this field is unspecified, the `machineTypeId` defaults to
      "n1-standard-1". This field is supported for Cloud Composer environments
      in versions composer-1.*.*-airflow-*.*.*.
    maxPodsPerNode: Optional. The maximum number of pods per node in the Cloud
      Composer GKE cluster. The value must be between 8 and 110 and it can be
      set only if the environment is VPC-native. The default value is 32.
      Values of this field will be propagated both to the `default-pool` node
      pool of the newly created GKE cluster, and to the default "Maximum Pods
      per Node" value which is used for newly created node pools if their
      value is not explicitly set during node pool creation. For more
      information, see [Optimizing IP address allocation]
      (https://cloud.google.com/kubernetes-engine/docs/how-to/flexible-pod-
      cidr). Cannot be updated. This field is supported for Cloud Composer
      environments in versions composer-1.*.*-airflow-*.*.*.
    network: Optional. The Compute Engine network to be used for machine
      communications, specified as a [relative resource
      name](/apis/design/resource_names#relative_resource_name). For example:
      "projects/{projectId}/global/networks/{networkId}". If unspecified, the
      default network in the environment's project is used. If a [Custom
      Subnet Network](/vpc/docs/vpc#vpc_networks_and_subnets) is provided,
      `nodeConfig.subnetwork` must also be provided. For [Shared
      VPC](/vpc/docs/shared-vpc) subnetwork requirements, see
      `nodeConfig.subnetwork`.
    oauthScopes: Optional. The set of Google API scopes to be made available
      on all node VMs. If `oauth_scopes` is empty, defaults to
      ["https://www.googleapis.com/auth/cloud-platform"]. Cannot be updated.
      This field is supported for Cloud Composer environments in versions
      composer-1.*.*-airflow-*.*.*.
    serviceAccount: Optional. The Google Cloud Platform Service Account to be
      used by the workloads. If a service account is not specified, the
      "default" Compute Engine service account is used. Cannot be updated.
    subnetwork: Optional. The Compute Engine subnetwork to be used for machine
      communications, specified as a [relative resource
      name](/apis/design/resource_names#relative_resource_name). For example:
      "projects/{projectId}/regions/{regionId}/subnetworks/{subnetworkId}" If
      a subnetwork is provided, `nodeConfig.network` must also be provided,
      and the subnetwork must belong to the enclosing environment's project
      and location.
    tags: Optional. The list of instance tags applied to all node VMs. Tags
      are used to identify valid sources or targets for network firewalls.
      Each tag within the list must comply with
      [RFC1035](https://www.ietf.org/rfc/rfc1035.txt). Cannot be updated. This
      field is supported for Cloud Composer environments in versions
      composer-1.*.*-airflow-*.*.*.
  """

  diskSizeGb = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  enableIpMasqAgent = _messages.BooleanField(2)
  ipAllocationPolicy = _messages.MessageField('IPAllocationPolicy', 3)
  location = _messages.StringField(4)
  machineType = _messages.StringField(5)
  maxPodsPerNode = _messages.IntegerField(6, variant=_messages.Variant.INT32)
  network = _messages.StringField(7)
  oauthScopes = _messages.StringField(8, repeated=True)
  serviceAccount = _messages.StringField(9)
  subnetwork = _messages.StringField(10)
  tags = _messages.StringField(11, repeated=True)


class Operation(_messages.Message):
  r"""This resource represents a long-running operation that is the result of
  a network API call.

  Messages:
    MetadataValue: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    ResponseValue: The normal response of the operation in case of success. If
      the original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

  Fields:
    done: If the value is `false`, it means the operation is still in
      progress. If `true`, the operation is completed, and either `error` or
      `response` is available.
    error: The error result of the operation in case of failure or
      cancellation.
    metadata: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    name: The server-assigned name, which is only unique within the same
      service that originally returns it. If you use the default HTTP mapping,
      the `name` should be a resource name ending with
      `operations/{unique_id}`.
    response: The normal response of the operation in case of success. If the
      original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MetadataValue(_messages.Message):
    r"""Service-specific metadata associated with the operation. It typically
    contains progress information and common metadata such as create time.
    Some services might not provide such metadata. Any method that returns a
    long-running operation should document the metadata type, if any.

    Messages:
      AdditionalProperty: An additional property for a MetadataValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ResponseValue(_messages.Message):
    r"""The normal response of the operation in case of success. If the
    original method returns no data on success, such as `Delete`, the response
    is `google.protobuf.Empty`. If the original method is standard
    `Get`/`Create`/`Update`, the response should be the resource. For other
    methods, the response should have the type `XxxResponse`, where `Xxx` is
    the original method name. For example, if the original method name is
    `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

    Messages:
      AdditionalProperty: An additional property for a ResponseValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ResponseValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  done = _messages.BooleanField(1)
  error = _messages.MessageField('Status', 2)
  metadata = _messages.MessageField('MetadataValue', 3)
  name = _messages.StringField(4)
  response = _messages.MessageField('ResponseValue', 5)


class OperationMetadata(_messages.Message):
  r"""Metadata describing an operation.

  Enums:
    OperationTypeValueValuesEnum: Output only. The type of operation being
      performed.
    StateValueValuesEnum: Output only. The current operation state.

  Fields:
    createTime: Output only. The time the operation was submitted to the
      server.
    endTime: Output only. The time when the operation terminated, regardless
      of its success. This field is unset if the operation is still ongoing.
    operationType: Output only. The type of operation being performed.
    resource: Output only. The resource being operated on, as a [relative
      resource name]( /apis/design/resource_names#relative_resource_name).
    resourceUuid: Output only. The UUID of the resource being operated on.
    state: Output only. The current operation state.
  """

  class OperationTypeValueValuesEnum(_messages.Enum):
    r"""Output only. The type of operation being performed.

    Values:
      TYPE_UNSPECIFIED: Unused.
      CREATE: A resource creation operation.
      DELETE: A resource deletion operation.
      UPDATE: A resource update operation.
      CHECK: A resource check operation.
      SAVE_SNAPSHOT: Saves snapshot of the resource operation.
      LOAD_SNAPSHOT: Loads snapshot of the resource operation.
    """
    TYPE_UNSPECIFIED = 0
    CREATE = 1
    DELETE = 2
    UPDATE = 3
    CHECK = 4
    SAVE_SNAPSHOT = 5
    LOAD_SNAPSHOT = 6

  class StateValueValuesEnum(_messages.Enum):
    r"""Output only. The current operation state.

    Values:
      STATE_UNSPECIFIED: Unused.
      PENDING: The operation has been created but is not yet started.
      RUNNING: The operation is underway.
      SUCCEEDED: The operation completed successfully.
      SUCCESSFUL: <no description>
      FAILED: The operation is no longer running but did not succeed.
    """
    STATE_UNSPECIFIED = 0
    PENDING = 1
    RUNNING = 2
    SUCCEEDED = 3
    SUCCESSFUL = 4
    FAILED = 5

  createTime = _messages.StringField(1)
  endTime = _messages.StringField(2)
  operationType = _messages.EnumField('OperationTypeValueValuesEnum', 3)
  resource = _messages.StringField(4)
  resourceUuid = _messages.StringField(5)
  state = _messages.EnumField('StateValueValuesEnum', 6)


class PrivateClusterConfig(_messages.Message):
  r"""Configuration options for the private GKE cluster in a Cloud Composer
  environment.

  Fields:
    enablePrivateEndpoint: Optional. If `true`, access to the public endpoint
      of the GKE cluster is denied.
    masterIpv4CidrBlock: Optional. The CIDR block from which IPv4 range for
      GKE master will be reserved. If left blank, the default value of
      '172.16.0.0/23' is used.
    masterIpv4ReservedRange: Output only. The IP range in CIDR notation to use
      for the hosted master network. This range is used for assigning internal
      IP addresses to the GKE cluster master or set of masters and to the
      internal load balancer virtual IP. This range must not overlap with any
      other ranges in use within the cluster's network.
  """

  enablePrivateEndpoint = _messages.BooleanField(1)
  masterIpv4CidrBlock = _messages.StringField(2)
  masterIpv4ReservedRange = _messages.StringField(3)


class PrivateEnvironmentConfig(_messages.Message):
  r"""The configuration information for configuring a Private IP Cloud
  Composer environment.

  Fields:
    cloudComposerConnectionSubnetwork: Optional. When specified, the
      environment will use Private Service Connect instead of VPC peerings to
      connect to Cloud SQL in the Tenant Project, and the PSC endpoint in the
      Customer Project will use an IP address from this subnetwork.
    cloudComposerNetworkIpv4CidrBlock: Optional. The CIDR block from which IP
      range for Cloud Composer Network in tenant project will be reserved.
      Needs to be disjoint from private_cluster_config.master_ipv4_cidr_block
      and cloud_sql_ipv4_cidr_block. This field is supported for Cloud
      Composer environments in versions composer-2.*.*-airflow-*.*.* and
      newer.
    cloudComposerNetworkIpv4ReservedRange: Output only. The IP range reserved
      for the tenant project's Cloud Composer network. This field is supported
      for Cloud Composer environments in versions composer-2.*.*-airflow-*.*.*
      and newer.
    cloudSqlIpv4CidrBlock: Optional. The CIDR block from which IP range in
      tenant project will be reserved for Cloud SQL. Needs to be disjoint from
      `web_server_ipv4_cidr_block`.
    enablePrivateEnvironment: Optional. If `true`, a Private IP Cloud Composer
      environment is created. If this field is set to true,
      `IPAllocationPolicy.use_ip_aliases` must be set to true for Cloud
      Composer environments in versions composer-1.*.*-airflow-*.*.*.
    enablePrivatelyUsedPublicIps: Optional. When enabled, IPs from public
      (non-RFC1918) ranges can be used for
      `IPAllocationPolicy.cluster_ipv4_cidr_block` and
      `IPAllocationPolicy.service_ipv4_cidr_block`.
    privateClusterConfig: Optional. Configuration for the private GKE cluster
      for a Private IP Cloud Composer environment.
    webServerIpv4CidrBlock: Optional. The CIDR block from which IP range for
      web server will be reserved. Needs to be disjoint from
      `private_cluster_config.master_ipv4_cidr_block` and
      `cloud_sql_ipv4_cidr_block`. This field is supported for Cloud Composer
      environments in versions composer-1.*.*-airflow-*.*.*.
    webServerIpv4ReservedRange: Output only. The IP range reserved for the
      tenant project's App Engine VMs. This field is supported for Cloud
      Composer environments in versions composer-1.*.*-airflow-*.*.*.
  """

  cloudComposerConnectionSubnetwork = _messages.StringField(1)
  cloudComposerNetworkIpv4CidrBlock = _messages.StringField(2)
  cloudComposerNetworkIpv4ReservedRange = _messages.StringField(3)
  cloudSqlIpv4CidrBlock = _messages.StringField(4)
  enablePrivateEnvironment = _messages.BooleanField(5)
  enablePrivatelyUsedPublicIps = _messages.BooleanField(6)
  privateClusterConfig = _messages.MessageField('PrivateClusterConfig', 7)
  webServerIpv4CidrBlock = _messages.StringField(8)
  webServerIpv4ReservedRange = _messages.StringField(9)


class RecoveryConfig(_messages.Message):
  r"""The Recovery settings of an environment.

  Fields:
    scheduledSnapshotsConfig: Optional. The configuration for scheduled
      snapshot creation mechanism.
    snapshotsLocation: Optional. The default Cloud Storage location for
      storing snapshots. Automatically created snapshots are stored in this
      location. If not set, the snapshots are stored in /snapshots folder in
      the environment's Cloud Storage bucket.
  """

  scheduledSnapshotsConfig = _messages.MessageField('ScheduledSnapshotsConfig', 1)
  snapshotsLocation = _messages.StringField(2)


class RestartWebServerRequest(_messages.Message):
  r"""Restart Airflow web server."""


class SaveSnapshotRequest(_messages.Message):
  r"""Request to create a snapshot of a Cloud Composer environment.

  Fields:
    snapshotLocation: Location in a Cloud Storage where the snapshot is going
      to be stored, e.g.: "gs://my-bucket/snapshots".
  """

  snapshotLocation = _messages.StringField(1)


class SaveSnapshotResponse(_messages.Message):
  r"""Response to SaveSnapshotRequest.

  Fields:
    snapshotPath: The fully-resolved Cloud Storage path of the created
      snapshot, e.g.: "gs://my-
      bucket/snapshots/project_location_environment_timestamp". This field is
      populated only if the snapshot creation was successful.
  """

  snapshotPath = _messages.StringField(1)


class ScheduledSnapshotsConfig(_messages.Message):
  r"""The configuration for scheduled snapshot creation mechanism.

  Fields:
    enabled: Optional. Whether scheduled snapshots creation is enabled.
    snapshotCreationSchedule: Optional. The cron expression representing the
      time when snapshots creation mechanism runs. This field is subject to
      additional validation around frequency of execution.
    snapshotLocation: Optional. The Cloud Storage location for storing
      automatically created snapshots.
    timeZone: Optional. Time zone that sets the context to interpret
      snapshot_creation_schedule.
  """

  enabled = _messages.BooleanField(1)
  snapshotCreationSchedule = _messages.StringField(2)
  snapshotLocation = _messages.StringField(3)
  timeZone = _messages.StringField(4)


class SchedulerResource(_messages.Message):
  r"""Configuration for resources used by Airflow schedulers.

  Fields:
    count: Optional. The number of schedulers.
    cpu: Optional. CPU request and limit for a single Airflow scheduler
      replica.
    memoryGb: Optional. Memory (GB) request and limit for a single Airflow
      scheduler replica.
    storageGb: Optional. Storage (GB) request and limit for a single Airflow
      scheduler replica.
  """

  count = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  cpu = _messages.FloatField(2, variant=_messages.Variant.FLOAT)
  memoryGb = _messages.FloatField(3, variant=_messages.Variant.FLOAT)
  storageGb = _messages.FloatField(4, variant=_messages.Variant.FLOAT)


class SoftwareConfig(_messages.Message):
  r"""Specifies the selection and configuration of software inside the
  environment.

  Enums:
    AirflowExecutorTypeValueValuesEnum: The `airflowExecutorType` specifies
      the [executor](https://airflow.apache.org/code.html?highlight=executor#e
      xecutors) by which task instances are run on Airflow. If this field is
      unspecified, the `airflowExecutorType` defaults to `celery`.

  Messages:
    AirflowConfigOverridesValue: Optional. Apache Airflow configuration
      properties to override. Property keys contain the section and property
      names, separated by a hyphen, for example "core-
      dags_are_paused_at_creation". Section names must not contain hyphens
      ("-"), opening square brackets ("["), or closing square brackets ("]").
      The property name must not be empty and must not contain an equals sign
      ("=") or semicolon (";"). Section and property names must not contain a
      period ("."). Apache Airflow configuration property names must be
      written in [snake_case](https://en.wikipedia.org/wiki/Snake_case).
      Property values can contain any character, and can be written in any
      lower/upper case format. Certain Apache Airflow configuration property
      values are [blocked](/composer/docs/concepts/airflow-configurations),
      and cannot be overridden.
    EnvVariablesValue: Optional. Additional environment variables to provide
      to the Apache Airflow scheduler, worker, and webserver processes.
      Environment variable names must match the regular expression `a-zA-Z_*`.
      They cannot specify Apache Airflow software configuration overrides
      (they cannot match the regular expression
      `AIRFLOW__[A-Z0-9_]+__[A-Z0-9_]+`), and they cannot match any of the
      following reserved names: * `AIRFLOW_HOME` * `C_FORCE_ROOT` *
      `CONTAINER_NAME` * `DAGS_FOLDER` * `GCP_PROJECT` * `GCS_BUCKET` *
      `GKE_CLUSTER_NAME` * `SQL_DATABASE` * `SQL_INSTANCE` * `SQL_PASSWORD` *
      `SQL_PROJECT` * `SQL_REGION` * `SQL_USER`
    PypiPackagesValue: Optional. Custom Python Package Index (PyPI) packages
      to be installed in the environment. Keys refer to the lowercase package
      name such as "numpy" and values are the lowercase extras and version
      specifier such as "==1.12.0", "[devel,gcp_api]", or "[devel]>=1.8.2,
      <1.9.2". To specify a package without pinning it to a version specifier,
      use the empty string as the value.

  Fields:
    airflowConfigOverrides: Optional. Apache Airflow configuration properties
      to override. Property keys contain the section and property names,
      separated by a hyphen, for example "core-dags_are_paused_at_creation".
      Section names must not contain hyphens ("-"), opening square brackets
      ("["), or closing square brackets ("]"). The property name must not be
      empty and must not contain an equals sign ("=") or semicolon (";").
      Section and property names must not contain a period ("."). Apache
      Airflow configuration property names must be written in
      [snake_case](https://en.wikipedia.org/wiki/Snake_case). Property values
      can contain any character, and can be written in any lower/upper case
      format. Certain Apache Airflow configuration property values are
      [blocked](/composer/docs/concepts/airflow-configurations), and cannot be
      overridden.
    airflowExecutorType: The `airflowExecutorType` specifies the [executor](ht
      tps://airflow.apache.org/code.html?highlight=executor#executors) by
      which task instances are run on Airflow. If this field is unspecified,
      the `airflowExecutorType` defaults to `celery`.
    cloudDataLineageIntegration: Optional. The configuration for Cloud Data
      Lineage integration.
    envVariables: Optional. Additional environment variables to provide to the
      Apache Airflow scheduler, worker, and webserver processes. Environment
      variable names must match the regular expression `a-zA-Z_*`. They cannot
      specify Apache Airflow software configuration overrides (they cannot
      match the regular expression `AIRFLOW__[A-Z0-9_]+__[A-Z0-9_]+`), and
      they cannot match any of the following reserved names: * `AIRFLOW_HOME`
      * `C_FORCE_ROOT` * `CONTAINER_NAME` * `DAGS_FOLDER` * `GCP_PROJECT` *
      `GCS_BUCKET` * `GKE_CLUSTER_NAME` * `SQL_DATABASE` * `SQL_INSTANCE` *
      `SQL_PASSWORD` * `SQL_PROJECT` * `SQL_REGION` * `SQL_USER`
    imageVersion: The version of the software running in the environment. This
      encapsulates both the version of Cloud Composer functionality and the
      version of Apache Airflow. It must match the regular expression `compose
      r-([0-9]+(\.[0-9]+\.[0-9]+(-preview\.[0-9]+)?)?|latest)-airflow-([0-9]+(
      \.[0-9]+(\.[0-9]+)?)?)`. When used as input, the server also checks if
      the provided version is supported and denies the request for an
      unsupported version. The Cloud Composer portion of the image version is
      a full [semantic version](https://semver.org), or an alias in the form
      of major version number or `latest`. When an alias is provided, the
      server replaces it with the current Cloud Composer version that
      satisfies the alias. The Apache Airflow portion of the image version is
      a full semantic version that points to one of the supported Apache
      Airflow versions, or an alias in the form of only major or major.minor
      versions specified. When an alias is provided, the server replaces it
      with the latest Apache Airflow version that satisfies the alias and is
      supported in the given Cloud Composer version. In all cases, the
      resolved image version is stored in the same field. See also [version
      list](/composer/docs/concepts/versioning/composer-versions) and
      [versioning overview](/composer/docs/concepts/versioning/composer-
      versioning-overview).
    pypiPackages: Optional. Custom Python Package Index (PyPI) packages to be
      installed in the environment. Keys refer to the lowercase package name
      such as "numpy" and values are the lowercase extras and version
      specifier such as "==1.12.0", "[devel,gcp_api]", or "[devel]>=1.8.2,
      <1.9.2". To specify a package without pinning it to a version specifier,
      use the empty string as the value.
    pythonVersion: Optional. The major version of Python used to run the
      Apache Airflow scheduler, worker, and webserver processes. Can be set to
      '2' or '3'. If not specified, the default is '3'. Cannot be updated.
      This field is only supported for Cloud Composer environments in versions
      composer-1.*.*-airflow-*.*.*. Environments in newer versions always use
      Python major version 3.
    schedulerCount: Optional. The number of schedulers for Airflow. This field
      is supported for Cloud Composer environments in versions
      composer-1.*.*-airflow-2.*.*.
  """

  class AirflowExecutorTypeValueValuesEnum(_messages.Enum):
    r"""The `airflowExecutorType` specifies the [executor](https://airflow.apa
    che.org/code.html?highlight=executor#executors) by which task instances
    are run on Airflow. If this field is unspecified, the
    `airflowExecutorType` defaults to `celery`.

    Values:
      AIRFLOW_EXECUTOR_TYPE_UNSPECIFIED: The Airflow executor type is
        unspecified.
      CELERY: The Celery executor will be used.
      KUBERNETES: The Kubernetes executor will be used.
    """
    AIRFLOW_EXECUTOR_TYPE_UNSPECIFIED = 0
    CELERY = 1
    KUBERNETES = 2

  @encoding.MapUnrecognizedFields('additionalProperties')
  class AirflowConfigOverridesValue(_messages.Message):
    r"""Optional. Apache Airflow configuration properties to override.
    Property keys contain the section and property names, separated by a
    hyphen, for example "core-dags_are_paused_at_creation". Section names must
    not contain hyphens ("-"), opening square brackets ("["), or closing
    square brackets ("]"). The property name must not be empty and must not
    contain an equals sign ("=") or semicolon (";"). Section and property
    names must not contain a period ("."). Apache Airflow configuration
    property names must be written in
    [snake_case](https://en.wikipedia.org/wiki/Snake_case). Property values
    can contain any character, and can be written in any lower/upper case
    format. Certain Apache Airflow configuration property values are
    [blocked](/composer/docs/concepts/airflow-configurations), and cannot be
    overridden.

    Messages:
      AdditionalProperty: An additional property for a
        AirflowConfigOverridesValue object.

    Fields:
      additionalProperties: Additional properties of type
        AirflowConfigOverridesValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a AirflowConfigOverridesValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class EnvVariablesValue(_messages.Message):
    r"""Optional. Additional environment variables to provide to the Apache
    Airflow scheduler, worker, and webserver processes. Environment variable
    names must match the regular expression `a-zA-Z_*`. They cannot specify
    Apache Airflow software configuration overrides (they cannot match the
    regular expression `AIRFLOW__[A-Z0-9_]+__[A-Z0-9_]+`), and they cannot
    match any of the following reserved names: * `AIRFLOW_HOME` *
    `C_FORCE_ROOT` * `CONTAINER_NAME` * `DAGS_FOLDER` * `GCP_PROJECT` *
    `GCS_BUCKET` * `GKE_CLUSTER_NAME` * `SQL_DATABASE` * `SQL_INSTANCE` *
    `SQL_PASSWORD` * `SQL_PROJECT` * `SQL_REGION` * `SQL_USER`

    Messages:
      AdditionalProperty: An additional property for a EnvVariablesValue
        object.

    Fields:
      additionalProperties: Additional properties of type EnvVariablesValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a EnvVariablesValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class PypiPackagesValue(_messages.Message):
    r"""Optional. Custom Python Package Index (PyPI) packages to be installed
    in the environment. Keys refer to the lowercase package name such as
    "numpy" and values are the lowercase extras and version specifier such as
    "==1.12.0", "[devel,gcp_api]", or "[devel]>=1.8.2, <1.9.2". To specify a
    package without pinning it to a version specifier, use the empty string as
    the value.

    Messages:
      AdditionalProperty: An additional property for a PypiPackagesValue
        object.

    Fields:
      additionalProperties: Additional properties of type PypiPackagesValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a PypiPackagesValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  airflowConfigOverrides = _messages.MessageField('AirflowConfigOverridesValue', 1)
  airflowExecutorType = _messages.EnumField('AirflowExecutorTypeValueValuesEnum', 2)
  cloudDataLineageIntegration = _messages.MessageField('CloudDataLineageIntegration', 3)
  envVariables = _messages.MessageField('EnvVariablesValue', 4)
  imageVersion = _messages.StringField(5)
  pypiPackages = _messages.MessageField('PypiPackagesValue', 6)
  pythonVersion = _messages.StringField(7)
  schedulerCount = _messages.IntegerField(8, variant=_messages.Variant.INT32)


class StandardQueryParameters(_messages.Message):
  r"""Query parameters accepted by all methods.

  Enums:
    FXgafvValueValuesEnum: V1 error format.
    AltValueValuesEnum: Data format for response.

  Fields:
    f__xgafv: V1 error format.
    access_token: OAuth access token.
    alt: Data format for response.
    callback: JSONP
    fields: Selector specifying which fields to include in a partial response.
    key: API key. Your API key identifies your project and provides you with
      API access, quota, and reports. Required unless you provide an OAuth 2.0
      token.
    oauth_token: OAuth 2.0 token for the current user.
    prettyPrint: Returns response with indentations and line breaks.
    quotaUser: Available to use for quota purposes for server-side
      applications. Can be any arbitrary string assigned to a user, but should
      not exceed 40 characters.
    trace: A tracing token of the form "token:<tokenid>" to include in api
      requests.
    uploadType: Legacy upload protocol for media (e.g. "media", "multipart").
    upload_protocol: Upload protocol for media (e.g. "raw", "multipart").
  """

  class AltValueValuesEnum(_messages.Enum):
    r"""Data format for response.

    Values:
      json: Responses with Content-Type of application/json
      media: Media download with context-dependent Content-Type
      proto: Responses with Content-Type of application/x-protobuf
    """
    json = 0
    media = 1
    proto = 2

  class FXgafvValueValuesEnum(_messages.Enum):
    r"""V1 error format.

    Values:
      _1: v1 error format
      _2: v2 error format
    """
    _1 = 0
    _2 = 1

  f__xgafv = _messages.EnumField('FXgafvValueValuesEnum', 1)
  access_token = _messages.StringField(2)
  alt = _messages.EnumField('AltValueValuesEnum', 3, default='json')
  callback = _messages.StringField(4)
  fields = _messages.StringField(5)
  key = _messages.StringField(6)
  oauth_token = _messages.StringField(7)
  prettyPrint = _messages.BooleanField(8, default=True)
  quotaUser = _messages.StringField(9)
  trace = _messages.StringField(10)
  uploadType = _messages.StringField(11)
  upload_protocol = _messages.StringField(12)


class Status(_messages.Message):
  r"""The `Status` type defines a logical error model that is suitable for
  different programming environments, including REST APIs and RPC APIs. It is
  used by [gRPC](https://github.com/grpc). Each `Status` message contains
  three pieces of data: error code, error message, and error details. You can
  find out more about this error model and how to work with it in the [API
  Design Guide](https://cloud.google.com/apis/design/errors).

  Messages:
    DetailsValueListEntry: A DetailsValueListEntry object.

  Fields:
    code: The status code, which should be an enum value of google.rpc.Code.
    details: A list of messages that carry the error details. There is a
      common set of message types for APIs to use.
    message: A developer-facing error message, which should be in English. Any
      user-facing error message should be localized and sent in the
      google.rpc.Status.details field, or localized by the client.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class DetailsValueListEntry(_messages.Message):
    r"""A DetailsValueListEntry object.

    Messages:
      AdditionalProperty: An additional property for a DetailsValueListEntry
        object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a DetailsValueListEntry object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  code = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  details = _messages.MessageField('DetailsValueListEntry', 2, repeated=True)
  message = _messages.StringField(3)


class Task(_messages.Message):
  r"""A single task in a DAG.

  Fields:
    dagId: The DAG ID of the DAG the task belongs to.
    dependsOnPast: When set to true, task instances will run sequentially
      while relying on the previous task's schedule to succeed.
    doXcomPush: if true, an XCom is pushed containing the Operator's result.
    downstreamTasks: IDs of downstream tasks from this task, i.e. those that
      wait on this task.
    emailOnFailure: Whether email alerts should be sent when a task failed.
    emailOnRetry: Whether email alerts should be sent when a task is retried.
    emails: Email addresses used in alerts.
    endDate: If specified, the scheduler won't go beyond this date.
    executionTimeout: Maximum time allowed for the execution of this task
      instance, if it goes beyond it will raise and fail.
    id: The task ID.
    maxRetryDelay: Maximum delay interval between retries.
    name: Required. The resource name of the task, in the form: "projects/{pro
      jectId}/locations/{locationId}/environments/{environmentId}/dags/{dagId}
      /tasks/{taskId}".
    owner: The owner of the task.
    pool: The slot pool this task should run in.
    poolSlots: The number of pool slots this task should use.
    priorityWeight: Priority weight of this task against other tasks.
    queue: Which queue to target when running this job.
    retries: The number of retries that should be performed before failing the
      task.
    retryDelay: Delay between retries.
    retryExponentialBackoff: Allow progressive longer waits between retries by
      using exponential backoff algorithm on retry delay.
    runAsUser: Unix username to impersonate while running the task.
    sla: Time by which the job is expected to succeed.
    startDate: Determines the execution_date for the first task instance.
    taskConcurrency: When set, a task will be able to limit the concurrent
      runs across execution dates.
    taskId: The task ID.
    taskType: The operator used in this task.
    triggerRule: Defines the rule by which dependencies are applied for the
      task to get triggered.
    upstreamTasks: IDs of upstream tasks from this task, i.e. those that this
      task waits on.
    waitForDownstream: When set to true, an instance of task X will wait for
      tasks immediately downstream of the previous instance of task X to
      finish successfully before it runs.
    weightRule: Weighting method used for the effective total priority weight
      of the task.
  """

  dagId = _messages.StringField(1)
  dependsOnPast = _messages.BooleanField(2)
  doXcomPush = _messages.BooleanField(3)
  downstreamTasks = _messages.StringField(4, repeated=True)
  emailOnFailure = _messages.BooleanField(5)
  emailOnRetry = _messages.BooleanField(6)
  emails = _messages.StringField(7, repeated=True)
  endDate = _messages.StringField(8)
  executionTimeout = _messages.StringField(9)
  id = _messages.StringField(10)
  maxRetryDelay = _messages.StringField(11)
  name = _messages.StringField(12)
  owner = _messages.StringField(13)
  pool = _messages.StringField(14)
  poolSlots = _messages.IntegerField(15, variant=_messages.Variant.INT32)
  priorityWeight = _messages.IntegerField(16, variant=_messages.Variant.INT32)
  queue = _messages.StringField(17)
  retries = _messages.IntegerField(18, variant=_messages.Variant.INT32)
  retryDelay = _messages.StringField(19)
  retryExponentialBackoff = _messages.BooleanField(20)
  runAsUser = _messages.StringField(21)
  sla = _messages.StringField(22)
  startDate = _messages.StringField(23)
  taskConcurrency = _messages.IntegerField(24, variant=_messages.Variant.INT32)
  taskId = _messages.StringField(25)
  taskType = _messages.StringField(26)
  triggerRule = _messages.StringField(27)
  upstreamTasks = _messages.StringField(28, repeated=True)
  waitForDownstream = _messages.BooleanField(29)
  weightRule = _messages.StringField(30)


class TaskInstance(_messages.Message):
  r"""Task instance in a DAG run.

  Enums:
    StateValueValuesEnum: Task instance state.

  Fields:
    dagId: The DAG ID of the DAG whose execution is described by the DAG run
      the taskInstance belongs to.
    dagRunId: The DAG run ID the task instance belongs to.
    endDate: Timestamp when the task instance finished execution.
    executionDate: Execution date for the task.
    hostname: Hostname of the machine or pod the task runs on.
    id: The task instance ID. It is the same as the task ID of a DAG.
    maxTries: The number of tries that should be performed before failing the
      task.
    name: Required. The resource name of the task instance, in the form: "proj
      ects/{projectId}/locations/{locationId}/environments/{environmentId}/dag
      s/{dagId}/dagRuns/{dagRunId}/taskInstances/{taskInstanceId}".
    pool: The slot pool this task runs in.
    priorityWeight: Priority weight of this task against other tasks.
    queue: Which queue to target when running this task.
    queuedDttm: Timestamp when the task was queued.
    startDate: Timestamp when the task instance started execution.
    state: Task instance state.
    taskId: The task instance ID. It is the same as the task ID in the DAG.
    taskType: The operator used in this task.
    tryNumber: The try number that this task number will be when it is
      actually run.
  """

  class StateValueValuesEnum(_messages.Enum):
    r"""Task instance state.

    Values:
      STATE_UNSPECIFIED: The state of the task instance is unknown.
      SUCCEEDED: Task execution succeeded.
      FAILED: Task execution failed.
      UPSTREAM_FAILED: Upstream task failed.
      SKIPPED: Task skipped.
      UP_FOR_RETRY: Task waiting to be retried.
      UP_FOR_RESCHEDULE: Task waiting to be rescheduled.
      QUEUED: Task queued.
      SCHEDULED: Task scheduled for execution.
      SENSING: Task in sensing mode.
      REMOVED: Task vanished from DAG before it ran.
      RUNNING: Task is executing.
      SHUTDOWN: External request to shut down (e.g. marked failed when
        running).
      RESTARTING: External request to restart (e.g. cleared when running).
      DEFERRED: Deferrable operator waiting on a trigger.
    """
    STATE_UNSPECIFIED = 0
    SUCCEEDED = 1
    FAILED = 2
    UPSTREAM_FAILED = 3
    SKIPPED = 4
    UP_FOR_RETRY = 5
    UP_FOR_RESCHEDULE = 6
    QUEUED = 7
    SCHEDULED = 8
    SENSING = 9
    REMOVED = 10
    RUNNING = 11
    SHUTDOWN = 12
    RESTARTING = 13
    DEFERRED = 14

  dagId = _messages.StringField(1)
  dagRunId = _messages.StringField(2)
  endDate = _messages.StringField(3)
  executionDate = _messages.StringField(4)
  hostname = _messages.StringField(5)
  id = _messages.StringField(6)
  maxTries = _messages.IntegerField(7, variant=_messages.Variant.INT32)
  name = _messages.StringField(8)
  pool = _messages.StringField(9)
  priorityWeight = _messages.IntegerField(10, variant=_messages.Variant.INT32)
  queue = _messages.StringField(11)
  queuedDttm = _messages.StringField(12)
  startDate = _messages.StringField(13)
  state = _messages.EnumField('StateValueValuesEnum', 14)
  taskId = _messages.StringField(15)
  taskType = _messages.StringField(16)
  tryNumber = _messages.IntegerField(17, variant=_messages.Variant.INT32)


class TriggerDagRequest(_messages.Message):
  r"""Request to trigger a DAG run.

  Messages:
    ConfValue: The key-value pairs get pickled into the conf attribute in the
      DAG run.

  Fields:
    conf: The key-value pairs get pickled into the conf attribute in the DAG
      run.
    dagRunId: The dag_run_id to be assigned to the triggered DAG run.
    executionDate: The execution date of the DAG run.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ConfValue(_messages.Message):
    r"""The key-value pairs get pickled into the conf attribute in the DAG
    run.

    Messages:
      AdditionalProperty: An additional property for a ConfValue object.

    Fields:
      additionalProperties: Properties of the object.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ConfValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  conf = _messages.MessageField('ConfValue', 1)
  dagRunId = _messages.StringField(2)
  executionDate = _messages.StringField(3)


class TriggererResource(_messages.Message):
  r"""Configuration for resources used by Airflow triggerers.

  Fields:
    count: Optional. The number of triggerers.
    cpu: Optional. CPU request and limit for a single Airflow triggerer
      replica.
    memoryGb: Optional. Memory (GB) request and limit for a single Airflow
      triggerer replica.
  """

  count = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  cpu = _messages.FloatField(2, variant=_messages.Variant.FLOAT)
  memoryGb = _messages.FloatField(3, variant=_messages.Variant.FLOAT)


class WebServerConfig(_messages.Message):
  r"""The configuration settings for the Airflow web server App Engine
  instance. Supported for Cloud Composer environments in versions
  composer-1.*.*-airflow-*.*.*.

  Fields:
    machineType: Optional. Machine type on which Airflow web server is
      running. It has to be one of: composer-n1-webserver-2,
      composer-n1-webserver-4 or composer-n1-webserver-8. If not specified,
      composer-n1-webserver-2 will be used. Value custom is returned only in
      response, if Airflow web server parameters were manually changed to a
      non-standard values.
  """

  machineType = _messages.StringField(1)


class WebServerNetworkAccessControl(_messages.Message):
  r"""Network-level access control policy for the Airflow web server.

  Fields:
    allowedIpRanges: A collection of allowed IP ranges with descriptions.
  """

  allowedIpRanges = _messages.MessageField('AllowedIpRange', 1, repeated=True)


class WebServerResource(_messages.Message):
  r"""Configuration for resources used by Airflow web server.

  Fields:
    cpu: Optional. CPU request and limit for Airflow web server.
    memoryGb: Optional. Memory (GB) request and limit for Airflow web server.
    storageGb: Optional. Storage (GB) request and limit for Airflow web
      server.
  """

  cpu = _messages.FloatField(1, variant=_messages.Variant.FLOAT)
  memoryGb = _messages.FloatField(2, variant=_messages.Variant.FLOAT)
  storageGb = _messages.FloatField(3, variant=_messages.Variant.FLOAT)


class WorkerResource(_messages.Message):
  r"""Configuration for resources used by Airflow workers.

  Fields:
    cpu: Optional. CPU request and limit for a single Airflow worker replica.
    maxCount: Optional. Maximum number of workers for autoscaling.
    memoryGb: Optional. Memory (GB) request and limit for a single Airflow
      worker replica.
    minCount: Optional. Minimum number of workers for autoscaling.
    storageGb: Optional. Storage (GB) request and limit for a single Airflow
      worker replica.
  """

  cpu = _messages.FloatField(1, variant=_messages.Variant.FLOAT)
  maxCount = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  memoryGb = _messages.FloatField(3, variant=_messages.Variant.FLOAT)
  minCount = _messages.IntegerField(4, variant=_messages.Variant.INT32)
  storageGb = _messages.FloatField(5, variant=_messages.Variant.FLOAT)


class WorkloadsConfig(_messages.Message):
  r"""The Kubernetes workloads configuration for GKE cluster associated with
  the Cloud Composer environment. Supported for Cloud Composer environments in
  versions composer-2.*.*-airflow-*.*.* and newer.

  Fields:
    scheduler: Optional. Resources used by Airflow scheduler.
    schedulerCpu: Optional. CPU request and limit for Airflow scheduler.
    triggerer: Optional. Resources used by Airflow triggerers.
    webServer: Optional. Resources used by Airflow web server.
    worker: Optional. Resources used by Airflow workers.
    workerCpu: Optional. CPU request and limit for Airflow worker.
    workerMaxCount: Optional. Maximum number of workers for autoscaling.
    workerMinCount: Optional. Minimum number of workers for autoscaling.
  """

  scheduler = _messages.MessageField('SchedulerResource', 1)
  schedulerCpu = _messages.FloatField(2, variant=_messages.Variant.FLOAT)
  triggerer = _messages.MessageField('TriggererResource', 3)
  webServer = _messages.MessageField('WebServerResource', 4)
  worker = _messages.MessageField('WorkerResource', 5)
  workerCpu = _messages.FloatField(6, variant=_messages.Variant.FLOAT)
  workerMaxCount = _messages.IntegerField(7, variant=_messages.Variant.INT32)
  workerMinCount = _messages.IntegerField(8, variant=_messages.Variant.INT32)


encoding.AddCustomJsonFieldMapping(
    StandardQueryParameters, 'f__xgafv', '$.xgafv')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_1', '1')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_2', '2')



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""CLI Utilities for beyondcorp app commands."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.util.args import labels_util
from googlecloudsdk.core import exceptions

MEMBER_PARSE_ERROR = ('Error parsing member [{}]: member must be prefixed of '
                      'the form serviceAccount:<value>.')
MEMBER_PREFIX = 'serviceAccount'


class MemberParseError(exceptions.Error):
  """Error if a member is not in correct format."""


def MemberProcessor(member):
  """Validates and parses a service account from member string.

  Expects string.

  Args:
    member: string in format of 'serviceAccount:<value>'.

  Raises:
    MemberParseError: if string is not in valid format 'serviceAccount:<value>',
    raises exception MemberParseError.

  Returns:
    string: Returns <value> part from 'serviceAccount:<value>'.
  """
  member_array = member.split(':')

  if len(member_array) == 2 and member_array[0] == MEMBER_PREFIX:
    return member_array[1]
  else:
    raise MemberParseError(
        MEMBER_PARSE_ERROR.format(member))


def UpdateLabelsFlags():
  """Defines flags for updating labels."""
  remove_group = base.ArgumentGroup(mutex=True)
  remove_group.AddArgument(labels_util.GetClearLabelsFlag())
  remove_group.AddArgument(labels_util.GetRemoveLabelsFlag(''))
  return [labels_util.GetUpdateLabelsFlag(''), remove_group]


def AddFieldToUpdateMask(field, patch_request):
  """Adds name of field to update mask."""
  update_mask = patch_request.updateMask
  if update_mask:
    if update_mask.count(field) == 0:
      patch_request.updateMask = update_mask + ',' + field
  else:
    patch_request.updateMask = field
  return patch_request



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utilities for `gcloud app firewall-rules`."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope import exceptions
from googlecloudsdk.core import resources
import six

# The default rule is placed at MaxInt32 - 1 and is always evaluated last
DEFAULT_RULE_PRIORITY = 2**31 - 1

LIST_FORMAT = """
  table(
    priority:sort=1,
    action,
    source_range,
    description
  )
  """

registry = resources.REGISTRY


def GetRegistry(version):
  global registry
  try:
    resources.REGISTRY.GetCollectionInfo('appengine', version)
  except resources.InvalidCollectionException:
    registry = resources.REGISTRY.Clone()
    registry.RegisterApiByName('appengine', version)
  return registry


def ParseFirewallRule(client, priority):
  """Creates a resource path given a firewall rule priority.

  Args:
    client: AppengineFirewallApiClient, the API client for this release track.
    priority: str, the priority of the rule.

  Returns:
    The resource for the rule.

  """
  res = GetRegistry(client.ApiVersion()).Parse(
      six.text_type(ParsePriority(priority)),
      params={'appsId': client.project},
      collection='appengine.apps.firewall.ingressRules')
  return res


def ParsePriority(priority):
  """Converts a priority to an integer."""
  if priority == 'default':
    priority = DEFAULT_RULE_PRIORITY

  try:
    priority_int = int(priority)
    if priority_int <= 0 or priority_int > DEFAULT_RULE_PRIORITY:
      raise exceptions.InvalidArgumentException(
          'priority', 'Priority must be between 1 and {0} inclusive.'.format(
              DEFAULT_RULE_PRIORITY))
    return priority_int
  except ValueError:
    raise exceptions.InvalidArgumentException(
        'priority', 'Priority should be an integer value or `default`.')


def ParseAction(messages, action):
  """Converts an action string to the corresponding enum value.

  Options are: 'allow' or 'deny', otherwise None will be returned.

  Args:
    messages: apitools.base.protorpclite.messages, the proto messages class for
      this API version for firewall.
    action: str, the action as a string
  Returns:
    ActionValueValuesEnum type
  """
  if not action:
    return None
  return messages.FirewallRule.ActionValueValuesEnum(action.upper())


def RaiseMinArgument():
  raise exceptions.MinimumArgumentException([
      '--action', '--source-range', '--description'
  ], 'Please specify at least one attribute to the firewall-rules update.')



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Implementation of update command for updating bucket settings."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.storage import flags
from googlecloudsdk.command_lib.storage import user_request_args_factory
from googlecloudsdk.command_lib.storage import wildcard_iterator
from googlecloudsdk.command_lib.storage.tasks import task_executor
from googlecloudsdk.command_lib.storage.tasks import task_graph_executor
from googlecloudsdk.command_lib.storage.tasks import task_status
from googlecloudsdk.command_lib.storage.tasks.buckets import update_bucket_task


_CORS_HELP_TEXT = """
Sets the Cross-Origin Resource Sharing (CORS) configuration on a bucket.
An example CORS JSON document looks like the following:

  [
    {
      "origin": ["http://origin1.example.com"],
      "responseHeader": ["Content-Type"],
      "method": ["GET"],
      "maxAgeSeconds": 3600
    }
  ]

For more information about supported endpoints for CORS, see
[Cloud Storage CORS support]
(https://cloud.google.com/storage/docs/cross-origin#server-side-support).
"""
_LABELS_HELP_TEXT = """
Sets the label configuration for the bucket. An example label JSON document
looks like the following:

  {
    "your_label_key": "your_label_value",
    "your_other_label_key": "your_other_label_value"
  }
"""
_LIFECYCLE_HELP_TEXT = """
Sets the lifecycle management configuration on a bucket. For example,
The following lifecycle management configuration JSON document
specifies that all objects in this bucket that are more than 365 days
old are deleted automatically:

  {
    "rule":
    [
      {
        "action": {"type": "Delete"},
        "condition": {"age": 365}
      }
    ]
  }
"""


def _add_common_args(parser):
  """Register flags for this command.

  Args:
    parser (argparse.ArgumentParser): The parser to add the arguments to.

  Returns:
    buckets update flag group
  """
  parser.add_argument(
      'url', nargs='+', type=str, help='URLs of the buckets to update.')
  cors = parser.add_mutually_exclusive_group()
  cors.add_argument('--cors-file', help=_CORS_HELP_TEXT)
  cors.add_argument(
      '--clear-cors',
      action='store_true',
      help="Clears the bucket's CORS settings.")
  parser.add_argument(
      '--default-storage-class',
      help='Sets the default storage class for the bucket.')
  default_encryption_key = parser.add_mutually_exclusive_group()
  default_encryption_key.add_argument(
      '--default-encryption-key',
      help='Set the default KMS key for the bucket.')
  default_encryption_key.add_argument(
      '--clear-default-encryption-key',
      action='store_true',
      help="Clears the bucket's default encryption key.")
  labels = parser.add_mutually_exclusive_group()
  labels.add_argument('--labels-file', help=_LABELS_HELP_TEXT)
  update_labels = labels.add_group()
  update_labels.add_argument(
      '--update-labels',
      metavar='LABEL_KEYS_AND_VALUES',
      type=arg_parsers.ArgDict(),
      help='Add or update labels. Example:'
      ' --update-labels=key1=value1,key2=value2')
  update_labels.add_argument(
      '--remove-labels',
      metavar='LABEL_KEYS',
      type=arg_parsers.ArgList(),
      help='Remove labels by their key names.')
  labels.add_argument(
      '--clear-labels',
      action='store_true',
      help='Clear all labels associated with a bucket.')
  lifecycle = parser.add_mutually_exclusive_group()
  lifecycle.add_argument('--lifecycle-file', help=_LIFECYCLE_HELP_TEXT)
  lifecycle.add_argument(
      '--clear-lifecycle',
      action='store_true',
      help='Removes all lifecycle configuration for the bucket.')
  log_bucket = parser.add_mutually_exclusive_group()
  log_bucket.add_argument(
      '--log-bucket',
      help='Enables usage logging of the bucket, outputting log files to the'
      " specified logging_bucket in this flag. Cloud Storage doesn't validate"
      ' the existence of the bucket receiving logs. In addition to enabling'
      ' logging on your bucket, you will also need to grant'
      ' cloud-storage-analytics@google.com write access to the log bucket.')
  log_bucket.add_argument(
      '--clear-log-bucket',
      action='store_true',
      help="Clears logging bucket receiving the usage current bucket's data.")
  log_object_prefix = parser.add_mutually_exclusive_group()
  log_object_prefix.add_argument(
      '--log-object-prefix',
      help='Specifies the object prefix for logging activity to the log'
      ' bucket. The default prefix is the bucket name. All read and write'
      ' activity to objects in the bucket will be logged for objects matching'
      ' the prefix.')
  log_object_prefix.add_argument(
      '--clear-log-object-prefix',
      action='store_true',
      help='Clears prefix used to determine what usage data to send to'
      ' logging bucket.')
  public_access_prevention = parser.add_mutually_exclusive_group()
  public_access_prevention.add_argument(
      '--public-access-prevention',
      '--pap',
      action=arg_parsers.StoreTrueFalseAction,
      help='If True, sets public access prevention to "enforced".'
      ' If False, sets public access prevention to "inherited".'
      ' For details on how exactly public access is blocked, see:'
      ' http://cloud/storage/docs/public-access-prevention')
  public_access_prevention.add_argument(
      '--clear-public-access-prevention',
      '--clear-pap',
      action='store_true',
      help='Unsets the public access prevention setting on a bucket.',
  )
  parser.add_argument(
      '--requester-pays',
      action=arg_parsers.StoreTrueFalseAction,
      help='Allows you to configure a Cloud Storage bucket so that the'
      ' requester pays all costs related to accessing the bucket and its'
      ' objects.')
  parser.add_argument(
      '--versioning',
      action=arg_parsers.StoreTrueFalseAction,
      help='Allows you to configure a Cloud Storage bucket to keep old'
      ' versions of objects.')
  web_main_page_suffix = parser.add_mutually_exclusive_group()
  web_main_page_suffix.add_argument(
      '--web-main-page-suffix',
      help='Cloud Storage allows you to configure a bucket to behave like a'
      ' static website. A subsequent GET bucket request through a custom'
      ' domain serves the specified "main" page instead of performing the'
      ' usual bucket listing.')
  web_main_page_suffix.add_argument(
      '--clear-web-main-page-suffix',
      action='store_true',
      help='Clear website main page suffix if bucket is hosting website.')
  web_error_page = parser.add_mutually_exclusive_group()
  web_error_page.add_argument(
      '--web-error-page',
      help='Cloud Storage allows you to configure a bucket to behave like a'
      ' static website. A subsequent GET bucket request through a custom'
      ' domain for a non-existent object serves the specified error page'
      ' instead of the standard Cloud Storage error.')
  web_error_page.add_argument(
      '--clear-web-error-page',
      action='store_true',
      help='Clear website error page if bucket is hosting website.')
  flags.add_continue_on_error_flag(parser)


def _add_alpha_args(parser):
  """Register flags for the alpha version of this command.

  Args:
    parser (argparse.ArgumentParser): The parser to add the arguments to.

  Returns:
    buckets update flag group
  """
  retention_period = parser.add_mutually_exclusive_group()
  retention_period.add_argument(
      '--retention-period',
      help='Minimum [retention period](https://cloud.google.com'
      '/storage/docs/bucket-lock#retention-periods)'
      ' for objects stored in the bucket, for example'
      ' ``--retention-period=1Y1M1D5S\'\'. Objects added to the bucket'
      ' cannot be deleted until they\'ve been stored for the specified'
      ' length of time. Default is no retention period. Only available'
      ' for Cloud Storage using the JSON API.')
  retention_period.add_argument(
      '--clear-retention-period',
      action='store_true',
      help='Clears the object retention period for a bucket.')
  parser.add_argument(
      '--lock-retention-period',
      action=arg_parsers.StoreTrueFalseAction,
      help='Locks an unlocked retention policy on the buckets. Caution: A'
      ' locked retention policy cannot be removed from a bucket or reduced in'
      ' duration. Once locked, deleting the bucket is the only way to'
      ' "remove" a retention policy.')
  parser.add_argument(
      '--default-event-based-hold',
      action=arg_parsers.StoreTrueFalseAction,
      help='Sets the default value for an event-based hold on the bucket.'
      ' By setting the default event-based hold on a bucket, newly-created'
      ' objects inherit that value as their event-based hold (it is not'
      ' applied retroactively).')
  parser.add_argument(
      '--uniform-bucket-level-access',
      action=arg_parsers.StoreTrueFalseAction,
      help='Enables or disables [uniform bucket-level access]'
      '(https://cloud.google.com/storage/docs/bucket-policy-only)'
      ' for the buckets.')


@base.ReleaseTracks(base.ReleaseTrack.GA)
class Update(base.Command):
  """Update bucket settings."""

  detailed_help = {
      'DESCRIPTION':
          """
      Update a bucket.
      """,
      'EXAMPLES':
          """

      The following command updates the default storage class of a Cloud Storage
      bucket named "my-bucket" to NEARLINE and sets requester pays to true:

        $ {command} gs://my-bucket --default-storage-class=NEARLINE --requester-pays
      """,
  }

  @staticmethod
  def Args(parser):
    _add_common_args(parser)

  def update_task_iterator(self, args):
    user_request_args = (
        user_request_args_factory.get_user_request_args_from_command_args(
            args, metadata_type=user_request_args_factory.MetadataType.BUCKET))
    for url in args.url:
      for resource in wildcard_iterator.get_wildcard_iterator(url):
        yield update_bucket_task.UpdateBucketTask(
            resource, user_request_args=user_request_args)

  def Run(self, args):
    task_status_queue = task_graph_executor.multiprocessing_context.Queue()
    self.exit_code = task_executor.execute_tasks(
        self.update_task_iterator(args),
        parallelizable=True,
        task_status_queue=task_status_queue,
        progress_manager_args=task_status.ProgressManagerArgs(
            increment_type=task_status.IncrementType.INTEGER,
            manifest_path=None),
        continue_on_error=args.continue_on_error,
    )


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class UpdateAlpha(Update):
  """Update bucket settings."""

  detailed_help = {
      'DESCRIPTION':
          """
      Update a bucket.
      """,
      'EXAMPLES':
          """

      The following command updates the retention period of a Cloud Storage
      bucket named "my-bucket" to one year and thirty-six minutes:

        $ {command} gs://my-bucket --retention-period=1y36m

      The following command clears the retention period of a bucket:

        $ {command} gs://my-bucket --clear-retention-period
      """,
  }

  @staticmethod
  def Args(parser):
    _add_common_args(parser)
    _add_alpha_args(parser)



# -*- coding: utf-8 -*- #
# Copyright 2013 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""A module to get an unauthenticated http object."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import enum
import google_auth_httplib2

from googlecloudsdk.core import context_aware
from googlecloudsdk.core import http_proxy
from googlecloudsdk.core import log
from googlecloudsdk.core import properties
from googlecloudsdk.core import transport
from googlecloudsdk.core.util import encoding

import httplib2
import six


def Http(timeout='unset', response_encoding=None, ca_certs=None):
  """Get an httplib2.Http client that is properly configured for use by gcloud.

  This method does not add credentials to the client.  For an Http client that
  has been authenticated, use core.credentials.http.Http().

  Args:
    timeout: double, The timeout in seconds to pass to httplib2.  This is the
        socket level timeout.  If timeout is None, timeout is infinite.  If
        default argument 'unset' is given, a sensible default is selected using
        transport.GetDefaultTimeout().
    response_encoding: str, the encoding to use to decode the response.
    ca_certs: str, absolute filename of a ca_certs file that overrides the
        default. The gcloud config property for ca_certs, in turn, overrides
        this argument.

  Returns:
    An httplib2.Http client object configured with all the required settings
    for gcloud.
  """
  http_client = _CreateRawHttpClient(timeout, ca_certs)
  http_client = RequestWrapper().WrapWithDefaults(http_client,
                                                  response_encoding)
  return http_client


def HttpClient(
    timeout=None,
    proxy_info=httplib2.proxy_info_from_environment,
    ca_certs=httplib2.CA_CERTS,
    disable_ssl_certificate_validation=False):
  """Returns a httplib2.Http subclass.

  Args:
    timeout: float, Request timeout, in seconds.
    proxy_info: httplib2.ProxyInfo object or callable
    ca_certs: str, absolute filename of a ca_certs file
    disable_ssl_certificate_validation: bool, If true, disable ssl certificate
        validation.

  Returns: A httplib2.Http subclass
  """
  if properties.VALUES.proxy.use_urllib3_via_shim.GetBool():
    import httplib2shim  # pylint:disable=g-import-not-at-top
    http_class = httplib2shim.Http
  else:
    http_class = httplib2.Http

  result = http_class(
      timeout=timeout,
      proxy_info=proxy_info,
      ca_certs=ca_certs,
      disable_ssl_certificate_validation=disable_ssl_certificate_validation)

  ca_config = context_aware.Config()
  if ca_config:
    log.debug('Using client certificate %s',
              ca_config.encrypted_client_cert_path)
    result.add_certificate(ca_config.encrypted_client_cert_path,
                           ca_config.encrypted_client_cert_path, '',
                           password=ca_config.encrypted_client_cert_password)

  return result


def _CreateRawHttpClient(timeout='unset', ca_certs=None):
  """Create an HTTP client matching the appropriate gcloud properties."""
  # Compared with setting the default timeout in the function signature (i.e.
  # timeout=300), this lets you test with short default timeouts by mocking
  # GetDefaultTimeout.
  if timeout != 'unset':
    effective_timeout = timeout
  else:
    effective_timeout = transport.GetDefaultTimeout()

  no_validate = properties.VALUES.auth.disable_ssl_validation.GetBool() or False
  ca_certs_property = properties.VALUES.core.custom_ca_certs_file.Get()
  # Believe an explicitly-set ca_certs property over anything we added.
  if ca_certs_property:
    ca_certs = ca_certs_property
  if no_validate:
    ca_certs = None
  return HttpClient(timeout=effective_timeout,
                    proxy_info=http_proxy.GetHttpProxyInfo(),
                    ca_certs=ca_certs,
                    disable_ssl_certificate_validation=no_validate)


class Request(transport.Request):
  """Encapsulates parameters for making a general HTTP request.

  This implementation does additional manipulation to ensure that the request
  parameters are specified in the same way as they were specified by the
  caller. That is, if the user calls:
      request('URI', 'GET', None, {'header': '1'})

  After modifying the request, we will call request using positional
  parameters, instead of transforming the request into:
      request('URI', method='GET', body=None, headers={'header': '1'})
  """

  @classmethod
  def FromRequestArgs(cls, *args, **kwargs):
    return cls(*args, **kwargs)

  def __init__(self, *args, **kwargs):
    self._args = args
    self._kwargs = kwargs

    uri = RequestParam.URI.Get(args, kwargs)
    if not six.PY2:
      # httplib2 needs text under Python 3.
      uri = encoding.Decode(uri)
    method = RequestParam.METHOD.Get(args, kwargs)
    headers = RequestParam.HEADERS.Get(args, kwargs) or {}
    body = RequestParam.BODY.Get(args, kwargs)
    super(Request, self).__init__(uri, method, headers, body)

  def ToRequestArgs(self):
    args, kwargs = list(self._args), dict(self._kwargs)
    RequestParam.URI.Set(args, kwargs, self.uri)
    if self.method:
      RequestParam.METHOD.Set(args, kwargs, self.method)
    if self.headers:
      RequestParam.HEADERS.Set(args, kwargs, self.headers)
    if self.body:
      RequestParam.BODY.Set(args, kwargs, self.body)
    return args, kwargs


class Response(transport.Response):
  """Encapsulates responses from making a general HTTP request."""

  @classmethod
  def FromResponse(cls, response):
    resp, content = response
    headers = {h: v for h, v in six.iteritems(resp) if h != 'status'}
    return cls(resp.get('status'), headers, content)


class RequestWrapper(transport.RequestWrapper):
  """Class for wrapping httplib.Httplib2 requests."""

  request_class = Request
  response_class = Response

  def DecodeResponse(self, response, response_encoding):
    response, content = response
    content = content.decode(response_encoding)
    return response, content


class RequestParam(enum.Enum):
  """Encapsulates parameters to a request() call and how to extract them.

  http.request has the following signature:
    request(self, uri, method="GET", body=None, headers=None, ...)
  """
  URI = ('uri', 0)
  METHOD = ('method', 1)
  BODY = ('body', 2)
  HEADERS = ('headers', 3)

  def __init__(self, arg_name, index):
    self.arg_name = arg_name
    self.index = index

  def Get(self, args, kwargs):
    if len(args) > self.index:
      return args[self.index]
    if self.arg_name in kwargs:
      return kwargs[self.arg_name]
    return None

  def Set(self, args, kwargs, value):
    if len(args) > self.index:
      args[self.index] = value
    else:
      kwargs[self.arg_name] = value


def GoogleAuthRequest():
  """A Request object for google-auth library.

  Returns:
    A http request which implements google.auth.transport.Request and uses
      gcloud's http object in the core.
  """
  return google_auth_httplib2.Request(Http())



# -*- coding: utf-8 -*- #
# Copyright 2019 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Shared resource flags for datafusion commands."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope.concepts import concepts
from googlecloudsdk.calliope.concepts import deps
from googlecloudsdk.command_lib.util.concepts import concept_parsers
from googlecloudsdk.core import properties


def LocationAttributeConfig():
  fallthroughs = [
      deps.PropertyFallthrough(properties.VALUES.datafusion.location)
  ]
  return concepts.ResourceParameterAttributeConfig(
      name='location',
      help_text='Compute Engine region in which to create the {resource}.',
      fallthroughs=fallthroughs)


def InstanceAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='instance',
      help_text='Cloud Data Fusion instance for the {resource}.')


def OperationAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='operation',
      help_text='Cloud Data Fusion operation for the {resource}.')


def GetLocationResourceSpec():
  return concepts.ResourceSpec(
      'datafusion.projects.locations',
      resource_name='location',
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      locationsId=LocationAttributeConfig())


def GetInstanceResourceSpec():
  return concepts.ResourceSpec(
      'datafusion.projects.locations.instances',
      resource_name='instance',
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      locationsId=LocationAttributeConfig(),
      instancesId=InstanceAttributeConfig())


def GetOperationResourceSpec():
  return concepts.ResourceSpec(
      'datafusion.projects.locations.operations',
      resource_name='operation',
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      locationsId=LocationAttributeConfig(),
      operationsId=OperationAttributeConfig())


def AddLocationResourceArg(parser, description):
  concept_parsers.ConceptParser.ForResource(
      '--location', GetLocationResourceSpec(), description,
      required=True).AddToParser(parser)


def AddInstanceResourceArg(parser, description):
  concept_parsers.ConceptParser.ForResource(
      'instance',
      GetInstanceResourceSpec(),
      description,
      required=True,
      plural=False).AddToParser(parser)


def AddOperationResourceArg(parser, description):
  concept_parsers.ConceptParser.ForResource(
      'operation', GetOperationResourceSpec(), description,
      required=True).AddToParser(parser)



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""gcloud dns managed-zone get-iam-policy command."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.dns import util
from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.dns import flags
from googlecloudsdk.command_lib.iam import iam_util


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class GetIamPolicyAlpha(base.Command):
  """Get the IAM policy for a Cloud DNS managed-zone.

  This command displays the IAM policy of the specified managed-zone.

  ## EXAMPLES

  To view the details of your managed-zone IAM policy , run:

    $ {command} my-zone
  """

  @staticmethod
  def Args(parser):
    flags.GetZoneResourceArg(
        'The name of the managed-zone to get the IAM policy for.').AddToParser(
            parser)

  def Run(self, args):
    api_version = util.GetApiFromTrack(self.ReleaseTrack())
    dns_client = util.GetApiClient(api_version)
    messages = apis.GetMessagesModule('dns', api_version)
    zone_ref = args.CONCEPTS.zone.Parse()
    resource_name = 'projects/{0}/managedZones/{1}'.format(
        zone_ref.project, zone_ref.managedZone)

    req = messages.DnsProjectsManagedZonesGetIamPolicyRequest(
        resource=resource_name,
        googleIamV1GetIamPolicyRequest=messages.GoogleIamV1GetIamPolicyRequest(
            options=messages.GoogleIamV1GetPolicyOptions(
                requestedPolicyVersion=iam_util
                .MAX_LIBRARY_IAM_SUPPORTED_VERSION)))

    return dns_client.projects_managedZones.GetIamPolicy(req)


@base.ReleaseTracks(base.ReleaseTrack.BETA)
class GetIamPolicyBeta(base.Command):
  """Get the IAM policy for a Cloud DNS managed-zone.

  This command displays the IAM policy of the specified managed-zone.

  ## EXAMPLES

  To view the details of your managed-zone IAM policy , run:

    $ {command} my-zone
  """

  @staticmethod
  def Args(parser):
    flags.GetZoneResourceArg(
        'The name of the managed-zone to get the IAM policy for.').AddToParser(
            parser)

  def Run(self, args):
    # The v1/v1beta2 apitools gcloud clients are not compatible with this method
    api_version = 'v2'
    dns_client = util.GetApiClient(api_version)
    messages = apis.GetMessagesModule('dns', api_version)
    zone_ref = args.CONCEPTS.zone.Parse()
    resource_name = 'projects/{0}/locations/{1}/managedZones/{2}'.format(
        zone_ref.project, 'global', zone_ref.managedZone)

    req = messages.DnsManagedZonesGetIamPolicyRequest(
        resource=resource_name,
        googleIamV1GetIamPolicyRequest=messages.GoogleIamV1GetIamPolicyRequest(
            options=messages.GoogleIamV1GetPolicyOptions(
                requestedPolicyVersion=iam_util
                .MAX_LIBRARY_IAM_SUPPORTED_VERSION)))

    return dns_client.managedZones.GetIamPolicy(req)



# -*- coding: utf-8 -*- #
# Copyright 2016 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Flags and helpers for the compute target-instances commands."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.command_lib.compute import completers as compute_completers
from googlecloudsdk.command_lib.compute import flags as compute_flags

DEFAULT_LIST_FORMAT = """\
    table(
      name,
      zone.basename(),
      instance.basename(),
      natPolicy
    )"""


class TargetInstancesCompleter(compute_completers.ListCommandCompleter):

  def __init__(self, **kwargs):
    super(TargetInstancesCompleter, self).__init__(
        collection='compute.targetInstances',
        list_command='compute target-instances list --uri',
        **kwargs)


def TargetInstanceArgument(required=True, plural=False):
  return compute_flags.ResourceArgument(
      resource_name='target instance',
      completer=TargetInstancesCompleter,
      plural=plural,
      required=required,
      zonal_collection='compute.targetInstances',
      zone_explanation=compute_flags.ZONE_PROPERTY_EXPLANATION)


NETWORK_ARG = compute_flags.ResourceArgument(
    name='--network',
    required=False,
    resource_name='network',
    global_collection='compute.networks',
    short_help='Network that this target instance applies to.',
    detailed_help="""\
        Network that this target instance applies to. This is only necessary if
        the corresponding instance has multiple network interfaces.
        If not specified, the default network interface will be used.
        """)


def AddNetwork(parser):
  NETWORK_ARG.AddArgument(parser)



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Functions for dealing with managed instances groups updates to standby policy."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.command_lib.compute.health_checks import flags as health_checks_flags

HEALTH_CHECK_ARG = health_checks_flags.HealthCheckArgument(
    '', '--health-check', required=False)

# Allow only up to 1 hour initial delay
_MAX_INITIAL_DELAY_DURATION = 3600
_MAX_INITIAL_DELAY_DURATION_HUMAN_READABLE = '1h'


def _InitialDelayValidator(value):
  duration_parser = arg_parsers.Duration(parsed_unit='s')
  parsed_value = duration_parser(value)
  if parsed_value > _MAX_INITIAL_DELAY_DURATION:
    raise arg_parsers.ArgumentTypeError(
        'The value of initial delay must be between 0 and {max_value}'.format(
            max_value=_MAX_INITIAL_DELAY_DURATION_HUMAN_READABLE))
  return parsed_value


def AddStandbyPolicyArgs(standby_policy_params):
  """Adds autohealing-related commandline arguments to parser."""
  standby_policy_params.add_argument(
      '--initial-delay',
      type=_InitialDelayValidator,
      help="""\
      Initialization delay before stopping or suspending instances
      in this managed instance group. For example: 5m or 300s.
      See `gcloud topic datetimes` for information on duration formats.
      """)



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utils for cluster maintenance window commands."""
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.edge_cloud.container import util
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.command_lib.run import flags


def ClearMaintenanceWindow(ref, args, request):
  """Clears cluster.maintenance_policy in the request if --clear-maintenance-window flag is specified.

  Args:
    ref: reference to the cluster object.
    args: command line arguments.
    request: API request to be issued

  Returns:
    modified request
  """

  del ref  # unused argument

  if not flags.FlagIsExplicitlySet(args, "clear_maintenance_window"):
    return request

  if not args.clear_maintenance_window:
    raise exceptions.BadArgumentException("--no-clear-maintenance-window",
                                          "The flag is not supported")
  if request.cluster is None:
    release_track = args.calliope_command.ReleaseTrack()
    request.cluster = util.GetMessagesModule(release_track).Cluster()
  request.cluster.maintenancePolicy = None

  _AddFieldToUpdateMask("maintenancePolicy", request)
  return request


def _AddFieldToUpdateMask(field, request):
  if not request.updateMask:
    request.updateMask = field
    return request

  if field not in request.updateMask:
    request.updateMask = request.updateMask + "," + field
  return request



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Common utility functions for sql export commands."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.sql import export_util
from googlecloudsdk.api_lib.sql import operations
from googlecloudsdk.api_lib.sql import validate
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.sql import flags
from googlecloudsdk.core import log
from googlecloudsdk.core import properties


def AddBaseExportFlags(
    parser,
    gz_supported=True,
    database_required=False,
    database_help_text=flags.DEFAULT_DATABASE_LIST_EXPORT_HELP_TEXT):
  """Adds the base export flags to the parser.

  Args:
    parser: The current argparse parser to add these flags to.
    gz_supported: Boolean, specifies whether gz compression is supported.
    database_required: Boolean, specifies whether the database flag is required.
    database_help_text: String, specifies the help text for the database flag.
  """
  base.ASYNC_FLAG.AddToParser(parser)
  flags.AddInstanceArgument(parser)

  uri_help_text = ('The path to the file in Google Cloud Storage where the '
                   'export will be stored. The URI is in the form '
                   'gs://bucketName/fileName. If the file already exists, the '
                   'operation fails.')
  if gz_supported:
    uri_help_text = uri_help_text + (' If the filename ends with .gz, the '
                                     'contents are compressed.')
  flags.AddUriArgument(parser, uri_help_text)
  flags.AddDatabaseList(parser, database_help_text, database_required)


def RunExportCommand(args, client, export_context):
  """Exports data from a Cloud SQL instance.

  Args:
    args: argparse.Namespace, The arguments that this command was invoked with.
    client: SqlClient instance, with sql_client and sql_messages props, for use
      in generating messages and making API calls.
    export_context: ExportContext; format-specific export metadata.

  Returns:
    A dict representing the export operation resource, if '--async' is used,
    or else None.

  Raises:
    HttpException: An HTTP error response was received while executing API
        request.
    ToolException: An error other than HTTP error occurred while executing the
        command.
  """
  sql_client = client.sql_client
  sql_messages = client.sql_messages

  validate.ValidateInstanceName(args.instance)
  instance_ref = client.resource_parser.Parse(
      args.instance,
      params={'project': properties.VALUES.core.project.GetOrFail},
      collection='sql.instances')

  export_request = sql_messages.SqlInstancesExportRequest(
      instance=instance_ref.instance,
      project=instance_ref.project,
      instancesExportRequest=sql_messages.InstancesExportRequest(
          exportContext=export_context))

  result_operation = sql_client.instances.Export(export_request)

  operation_ref = client.resource_parser.Create(
      'sql.operations',
      operation=result_operation.name,
      project=instance_ref.project)

  if args.async_:
    return sql_client.operations.Get(
        sql_messages.SqlOperationsGetRequest(
            project=operation_ref.project, operation=operation_ref.operation))

  operations.OperationsV1Beta4.WaitForOperation(sql_client, operation_ref,
                                                'Exporting Cloud SQL instance')

  log.status.write('Exported [{instance}] to [{bucket}].\n'.format(
      instance=instance_ref, bucket=args.uri))

  return None


def RunSqlExportCommand(args, client):
  """Exports data from a Cloud SQL instance to a MySQL dump file.

  Args:
    args: argparse.Namespace, The arguments that this command was invoked with.
    client: SqlClient instance, with sql_client and sql_messages props, for use
      in generating messages and making API calls.

  Returns:
    A dict object representing the operations resource describing the export
    operation if the export was successful.
  """
  sql_export_context = export_util.SqlExportContext(
      client.sql_messages,
      args.uri,
      args.database,
      args.table,
      offload=args.offload)
  if args.offload:
    log.status.write(
        'Serverless exports cost extra. See the pricing page for more information: https://cloud.google.com/sql/pricing.\n'
    )
  return RunExportCommand(args, client, sql_export_context)


def RunCsvExportCommand(args, client):
  """Exports data from a Cloud SQL instance to a CSV file.

  Args:
    args: argparse.Namespace, The arguments that this command was invoked with.
    client: SqlClient instance, with sql_client and sql_messages props, for use
      in generating messages and making API calls.

  Returns:
    A dict object representing the operations resource describing the export
    operation if the export was successful.
  """
  csv_export_context = export_util.CsvExportContext(
      client.sql_messages,
      args.uri,
      args.database,
      args.query,
      offload=args.offload,
      quote=args.quote,
      escape=args.escape,
      fields_terminated_by=args.fields_terminated_by,
      lines_terminated_by=args.lines_terminated_by)
  if args.offload:
    log.status.write(
        'Serverless exports cost extra. See the pricing page for more information: https://cloud.google.com/sql/pricing.\n'
    )
  return RunExportCommand(args, client, csv_export_context)


def RunBakExportCommand(args, client):
  """Export data from a Cloud SQL instance to a SQL Server BAK file.

  Args:
    args: argparse.Namespace, The arguments that this command was invoked with.
    client: SqlClient instance, with sql_client and sql_messages props, for use
      in generating messages and making API calls.

  Returns:
    A dict object representing the operations resource describing the export
    operation if the export was successful.
  """
  sql_export_context = export_util.BakExportContext(client.sql_messages,
                                                    args.uri, args.database)
  return RunExportCommand(args, client, sql_export_context)



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Task for uploading small files in one shot."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.storage import api_factory
from googlecloudsdk.api_lib.storage import cloud_api
from googlecloudsdk.api_lib.storage import request_config_factory
from googlecloudsdk.command_lib.storage.tasks.cp import upload_task
from googlecloudsdk.command_lib.storage.tasks.cp import upload_util


class SimpleUploadTask(upload_task.UploadTask):
  """Uploads a file with a non-resumable strategy.

  Not suitable for composite uploads, which require resumable upload tracker
  files to correctly clean up temporary components.
  """

  def execute(self, task_status_queue=None):
    """Performs a simple upload. See base class for information on args."""
    api = api_factory.get_api(self._destination_resource.storage_url.scheme)
    request_config = request_config_factory.get_request_config(
        self._destination_resource.storage_url,
        content_type=upload_util.get_content_type(
            self._source_resource.storage_url.object_name,
            self._source_resource.storage_url.is_stream),
        md5_hash=self._source_resource.md5_hash,
        size=self._length)

    digesters = upload_util.get_digesters(
        self._source_resource,
        self._destination_resource)
    source_stream = upload_util.get_stream(
        self._source_resource,
        length=self._length,
        digesters=digesters,
        task_status_queue=task_status_queue,
        destination_resource=self._destination_resource)

    with source_stream:
      uploaded_object_resource = api.upload_object(
          source_stream,
          self._destination_resource,
          request_config,
          source_resource=self._source_resource,
          upload_strategy=cloud_api.UploadStrategy.SIMPLE)

    upload_util.validate_uploaded_object(
        digesters,
        uploaded_object_resource,
        task_status_queue)
